@@ -2,8 +2,12 @@
 automatically."""
 
 import logging
+import itertools
 import re
 
+from ..compat import StringIO
+
+from mako.pygen import PythonPrinter
 from sqlalchemy.engine.reflection import Inspector
 from sqlalchemy.util import OrderedSet
 from .compare import _compare_tables
@@ -138,13 +142,38 @@ def _produce_migration_diffs(context, template_args,
     diffs = []
     _produce_net_changes(connection, metadata, diffs,
                          autogen_context, object_filters, include_schemas)
-    template_args[opts['upgrade_token']] = \
-        _indent(_produce_upgrade_commands(diffs, autogen_context))
-    template_args[opts['downgrade_token']] = \
-        _indent(_produce_downgrade_commands(diffs, autogen_context))
+
+    template_args[opts['upgrade_token']] = _indent(_render_cmd_body(
+        _produce_upgrade_commands, diffs, autogen_context))
+    template_args[opts['downgrade_token']] = _indent(_render_cmd_body(
+        _produce_downgrade_commands, diffs, autogen_context))
     template_args['imports'] = "\n".join(sorted(imports))
 
 
+def _indent(text):
+    text = re.compile(r'^', re.M).sub("    ", text).strip()
+    text = re.compile(r' +$', re.M).sub("", text)
+    return text
+
+
+def _render_cmd_body(fn, diffs, autogen_context):
+
+    buf = StringIO()
+    printer = PythonPrinter(buf)
+
+    printer.writeline(
+        "### commands auto generated by Alembic - "
+        "please adjust! ###"
+    )
+
+    for line in fn(diffs, autogen_context):
+        printer.writeline(line)
+
+    printer.writeline("### end Alembic commands ###")
+
+    return buf.getvalue()
+
+
 def _get_object_filters(
         context_opts, include_symbol=None, include_object=None):
     include_symbol = context_opts.get('include_symbol', include_symbol)
@@ -176,13 +205,6 @@ def _autogen_context(context, imports):
     }, connection
 
 
-def _indent(text):
-    text = "### commands auto generated by Alembic - "\
-        "please adjust! ###\n" + text
-    text += "\n### end Alembic commands ###"
-    text = re.compile(r'^', re.M).sub("    ", text).strip()
-    return text
-
 ###################################################
 # walk structures
 
@@ -191,7 +213,6 @@ def _produce_net_changes(connection, metadata, diffs, autogen_context,
                          object_filters=(),
                          include_schemas=False):
     inspector = Inspector.from_engine(connection)
-    # TODO: not hardcode alembic_version here ?
     conn_table_names = set()
 
     default_schema = connection.dialect.default_schema_name
@@ -205,13 +226,20 @@ def _produce_net_changes(connection, metadata, diffs, autogen_context,
     else:
         schemas = [None]
 
+    version_table_schema = autogen_context['context'].version_table_schema
+    version_table = autogen_context['context'].version_table
+
     for s in schemas:
-        tables = set(inspector.get_table_names(schema=s)).\
-            difference(['alembic_version'])
+        tables = set(inspector.get_table_names(schema=s))
+        if s == version_table_schema:
+            tables = tables.difference(
+                [autogen_context['context'].version_table]
+            )
         conn_table_names.update(zip([s] * len(tables), tables))
 
-    metadata_table_names = OrderedSet([(table.schema, table.name)
-                                       for table in metadata.sorted_tables])
+    metadata_table_names = OrderedSet(
+        [(table.schema, table.name) for table in metadata.sorted_tables]
+    ).difference([(version_table_schema, version_table)])
 
     _compare_tables(conn_table_names, metadata_table_names,
                     object_filters,
@@ -219,21 +247,32 @@ def _produce_net_changes(connection, metadata, diffs, autogen_context,
 
 
 def _produce_upgrade_commands(diffs, autogen_context):
-    buf = []
-    for diff in diffs:
-        buf.append(_invoke_command("upgrade", diff, autogen_context))
-    if not buf:
-        buf = ["pass"]
-    return "\n".join(buf)
+    return _produce_commands("upgrade", diffs, autogen_context)
 
 
 def _produce_downgrade_commands(diffs, autogen_context):
-    buf = []
-    for diff in reversed(diffs):
-        buf.append(_invoke_command("downgrade", diff, autogen_context))
-    if not buf:
-        buf = ["pass"]
-    return "\n".join(buf)
+    return _produce_commands("downgrade", diffs, autogen_context)
+
+
+def _produce_commands(type_, diffs, autogen_context):
+    opts = autogen_context['opts']
+    render_as_batch = opts.get('render_as_batch', False)
+
+    if diffs:
+        if type_ == 'downgrade':
+            diffs = reversed(diffs)
+        for (schema, table), subdiffs in _group_diffs_by_table(diffs):
+            if table is not None and render_as_batch:
+                yield "with op.batch_alter_table"\
+                    "(%r, schema=%r) as batch_op:" % (table, schema)
+                autogen_context['batch_prefix'] = 'batch_op.'
+            for diff in subdiffs:
+                yield _invoke_command(type_, diff, autogen_context)
+            if table is not None and render_as_batch:
+                del autogen_context['batch_prefix']
+                yield ""
+    else:
+        yield "pass"
 
 
 def _invoke_command(updown, args, autogen_context):
@@ -297,3 +336,23 @@ def _invoke_modify_command(updown, args, autogen_context):
     if "server_default" in kw:
         kw.pop("existing_server_default", None)
     return _modify_col(tname, cname, autogen_context, schema=sname, **kw)
+
+
+def _group_diffs_by_table(diffs):
+    _adddrop = {
+        "table": lambda diff: (None, None),
+        "column": lambda diff: (diff[0], diff[1]),
+        "index": lambda diff: (diff[0].table.schema, diff[0].table.name),
+        "constraint": lambda diff: (diff[0].table.schema, diff[0].table.name)
+    }
+
+    def _derive_table(diff):
+        if isinstance(diff, tuple):
+            cmd_type = diff[0]
+            adddrop, cmd_type = cmd_type.split("_")
+            return _adddrop[cmd_type](diff[1:])
+        else:
+            sname, tname = diff[0][1:3]
+            return sname, tname
+
+    return itertools.groupby(diffs, _derive_table)
