@@ -1,9 +1,11 @@
 from sqlalchemy import schema as sa_schema, types as sqltypes
+from sqlalchemy import event
 import logging
 from .. import compat
-from .render import _render_server_default
 from sqlalchemy.util import OrderedSet
-
+import re
+from .render import _user_defined_render
+import contextlib
 
 log = logging.getLogger(__name__)
 
@@ -63,7 +65,13 @@ def _compare_tables(conn_table_names, metadata_table_names,
         name = sa_schema._get_table_key(tname, s)
         exists = name in removal_metadata.tables
         t = sa_schema.Table(tname, removal_metadata, schema=s)
+
         if not exists:
+            event.listen(
+                t,
+                "column_reflect",
+                autogen_context['context'].impl.
+                _compat_autogen_column_reflect(inspector))
             inspector.reflecttable(t, None)
         if _run_filters(t, tname, "table", True, None, object_filters):
             diffs.append(("remove_table", t))
@@ -78,6 +86,11 @@ def _compare_tables(conn_table_names, metadata_table_names,
         exists = name in existing_metadata.tables
         t = sa_schema.Table(tname, existing_metadata, schema=s)
         if not exists:
+            event.listen(
+                t,
+                "column_reflect",
+                autogen_context['context'].impl.
+                _compat_autogen_column_reflect(inspector))
             inspector.reflecttable(t, None)
         conn_column_info[(s, tname)] = t
 
@@ -90,14 +103,15 @@ def _compare_tables(conn_table_names, metadata_table_names,
         if _run_filters(
                 metadata_table, tname, "table", False,
                 conn_table, object_filters):
-            _compare_columns(s, tname, object_filters,
-                             conn_table,
-                             metadata_table,
-                             diffs, autogen_context, inspector)
-            _compare_indexes_and_uniques(s, tname, object_filters,
-                                         conn_table,
-                                         metadata_table,
-                                         diffs, autogen_context, inspector)
+            with _compare_columns(
+                s, tname, object_filters,
+                conn_table,
+                metadata_table,
+                    diffs, autogen_context, inspector):
+                _compare_indexes_and_uniques(s, tname, object_filters,
+                                             conn_table,
+                                             metadata_table,
+                                             diffs, autogen_context, inspector)
 
     # TODO:
     # table constraints
@@ -105,6 +119,7 @@ def _compare_tables(conn_table_names, metadata_table_names,
 
 
 def _make_index(params, conn_table):
+    # TODO: add .info such as 'duplicates_constraint'
     return sa_schema.Index(
         params['name'],
         *[conn_table.c[cname] for cname in params['column_names']],
@@ -113,12 +128,14 @@ def _make_index(params, conn_table):
 
 
 def _make_unique_constraint(params, conn_table):
+    # TODO: add .info such as 'duplicates_index'
     return sa_schema.UniqueConstraint(
         *[conn_table.c[cname] for cname in params['column_names']],
         name=params['name']
     )
 
 
+@contextlib.contextmanager
 def _compare_columns(schema, tname, object_filters, conn_table, metadata_table,
                      diffs, autogen_context, inspector):
     name = '%s.%s' % (schema, tname) if schema else tname
@@ -134,14 +151,6 @@ def _compare_columns(schema, tname, object_filters, conn_table, metadata_table,
             )
             log.info("Detected added column '%s.%s'", name, cname)
 
-    for cname in set(conn_col_names).difference(metadata_col_names):
-        if _run_filters(conn_table.c[cname], cname,
-                        "column", True, None, object_filters):
-            diffs.append(
-                ("remove_column", schema, tname, conn_table.c[cname])
-            )
-            log.info("Detected removed column '%s.%s'", name, cname)
-
     for colname in metadata_col_names.intersection(conn_col_names):
         metadata_col = metadata_cols_by_name[colname]
         conn_col = conn_table.c[colname]
@@ -155,11 +164,13 @@ def _compare_columns(schema, tname, object_filters, conn_table, metadata_table,
                       metadata_col,
                       col_diff, autogen_context
                       )
-        _compare_nullable(schema, tname, colname,
-                          conn_col,
-                          metadata_col.nullable,
-                          col_diff, autogen_context
-                          )
+        # work around SQLAlchemy issue #3023
+        if not metadata_col.primary_key:
+            _compare_nullable(schema, tname, colname,
+                              conn_col,
+                              metadata_col.nullable,
+                              col_diff, autogen_context
+                              )
         _compare_server_default(schema, tname, colname,
                                 conn_col,
                                 metadata_col,
@@ -168,6 +179,17 @@ def _compare_columns(schema, tname, object_filters, conn_table, metadata_table,
         if col_diff:
             diffs.append(col_diff)
 
+    yield
+
+    for cname in set(conn_col_names).difference(metadata_col_names):
+        if _run_filters(conn_table.c[cname], cname,
+                        "column", True, None, object_filters):
+            diffs.append(
+                ("remove_column", schema, tname, conn_table.c[cname])
+            )
+            log.info("Detected removed column '%s.%s'", name, cname)
+
+
 
 class _constraint_sig(object):
 
@@ -317,12 +339,13 @@ def _compare_indexes_and_uniques(schema, tname, object_filters, conn_table,
 
     def obj_added(obj):
         if obj.is_index:
-            diffs.append(("add_index", obj.const))
-            log.info("Detected added index '%s' on %s",
-                     obj.name, ', '.join([
-                         "'%s'" % obj.column_names
-                     ])
-                     )
+            if _run_filters(
+                    obj.const, obj.name, "index", False, None, object_filters):
+                diffs.append(("add_index", obj.const))
+                log.info("Detected added index '%s' on %s",
+                         obj.name, ', '.join([
+                             "'%s'" % obj.column_names
+                         ]))
         else:
             if not supports_unique_constraints:
                 # can't report unique indexes as added if we don't
@@ -331,12 +354,14 @@ def _compare_indexes_and_uniques(schema, tname, object_filters, conn_table,
             if is_create_table:
                 # unique constraints are created inline with table defs
                 return
-            diffs.append(("add_constraint", obj.const))
-            log.info("Detected added unique constraint '%s' on %s",
-                     obj.name, ', '.join([
-                         "'%s'" % obj.column_names
-                     ])
-                     )
+            if _run_filters(
+                    obj.const, obj.name,
+                    "unique_constraint", False, None, object_filters):
+                diffs.append(("add_constraint", obj.const))
+                log.info("Detected added unique constraint '%s' on %s",
+                         obj.name, ', '.join([
+                             "'%s'" % obj.column_names
+                         ]))
 
     def obj_removed(obj):
         if obj.is_index:
@@ -346,27 +371,39 @@ def _compare_indexes_and_uniques(schema, tname, object_filters, conn_table,
                 # be sure what we're doing here
                 return
 
-            diffs.append(("remove_index", obj.const))
-            log.info("Detected removed index '%s' on '%s'", obj.name, tname)
+            if _run_filters(
+                    obj.const, obj.name, "index", True, None, object_filters):
+                diffs.append(("remove_index", obj.const))
+                log.info(
+                    "Detected removed index '%s' on '%s'", obj.name, tname)
         else:
-            diffs.append(("remove_constraint", obj.const))
-            log.info("Detected removed unique constraint '%s' on '%s'",
-                     obj.name, tname
-                     )
+            if _run_filters(
+                    obj.const, obj.name,
+                    "unique_constraint", True, None, object_filters):
+                diffs.append(("remove_constraint", obj.const))
+                log.info("Detected removed unique constraint '%s' on '%s'",
+                         obj.name, tname
+                         )
 
     def obj_changed(old, new, msg):
         if old.is_index:
-            log.info("Detected changed index '%s' on '%s':%s",
-                     old.name, tname, ', '.join(msg)
-                     )
-            diffs.append(("remove_index", old.const))
-            diffs.append(("add_index", new.const))
+            if _run_filters(
+                    new.const, new.name, "index",
+                    False, old.const, object_filters):
+                log.info("Detected changed index '%s' on '%s':%s",
+                         old.name, tname, ', '.join(msg)
+                         )
+                diffs.append(("remove_index", old.const))
+                diffs.append(("add_index", new.const))
         else:
-            log.info("Detected changed unique constraint '%s' on '%s':%s",
-                     old.name, tname, ', '.join(msg)
-                     )
-            diffs.append(("remove_constraint", old.const))
-            diffs.append(("add_constraint", new.const))
+            if _run_filters(
+                    new.const, new.name,
+                    "unique_constraint", False, old.const, object_filters):
+                log.info("Detected changed unique constraint '%s' on '%s':%s",
+                         old.name, tname, ', '.join(msg)
+                         )
+                diffs.append(("remove_constraint", old.const))
+                diffs.append(("add_constraint", new.const))
 
     for added_name in sorted(set(metadata_names).difference(conn_names)):
         obj = metadata_names[added_name]
@@ -475,9 +512,25 @@ def _compare_type(schema, tname, cname, conn_col,
 
 def _render_server_default_for_compare(metadata_default,
                                        metadata_col, autogen_context):
-    return _render_server_default(
-        metadata_default, autogen_context,
-        repr_=metadata_col.type._type_affinity is sqltypes.String)
+    rendered = _user_defined_render(
+        "server_default", metadata_default, autogen_context)
+    if rendered is not False:
+        return rendered
+
+    if isinstance(metadata_default, sa_schema.DefaultClause):
+        if isinstance(metadata_default.arg, compat.string_types):
+            metadata_default = metadata_default.arg
+        else:
+            metadata_default = str(metadata_default.arg.compile(
+                dialect=autogen_context['dialect']))
+    if isinstance(metadata_default, compat.string_types):
+        if metadata_col.type._type_affinity is sqltypes.String:
+            metadata_default = re.sub(r"^'|'$", "", metadata_default)
+            return repr(metadata_default)
+        else:
+            return metadata_default
+    else:
+        return None
 
 
 def _compare_server_default(schema, tname, cname, conn_col, metadata_col,
@@ -492,6 +545,7 @@ def _compare_server_default(schema, tname, cname, conn_col, metadata_col,
 
     rendered_conn_default = conn_col.server_default.arg.text \
         if conn_col.server_default else None
+
     isdiff = autogen_context['context']._compare_server_default(
         conn_col, metadata_col,
         rendered_metadata_default,
