@@ -28,19 +28,50 @@ class _f_name(object):
         self.name = name
 
     def __repr__(self):
-        return "%sf(%r)" % (self.prefix, self.name)
+        return "%sf(%r)" % (self.prefix, _ident(self.name))
 
 
-def _render_potential_expr(value, autogen_context):
+def _ident(name):
+    """produce a __repr__() object for a string identifier that may
+    use quoted_name() in SQLAlchemy 0.9 and greater.
+
+    The issue worked around here is that quoted_name() doesn't have
+    very good repr() behavior by itself when unicode is involved.
+
+    """
+    if name is None:
+        return name
+    elif compat.sqla_09 and isinstance(name, sql.elements.quoted_name):
+        if compat.py2k:
+            # the attempt to encode to ascii here isn't super ideal,
+            # however we are trying to cut down on an explosion of
+            # u'' literals only when py2k + SQLA 0.9, in particular
+            # makes unit tests testing code generation very difficult
+            try:
+                return name.encode('ascii')
+            except UnicodeError:
+                return compat.text_type(name)
+        else:
+            return compat.text_type(name)
+    elif isinstance(name, compat.string_types):
+        return name
+
+
+def _render_potential_expr(value, autogen_context, wrap_in_text=True):
     if isinstance(value, sql.ClauseElement):
         if compat.sqla_08:
             compile_kw = dict(compile_kwargs={'literal_binds': True})
         else:
             compile_kw = {}
 
-        return "%(prefix)stext(%(sql)r)" % {
+        if wrap_in_text:
+            template = "%(prefix)stext(%(sql)r)"
+        else:
+            template = "%(sql)r"
+
+        return template % {
             "prefix": _sqlalchemy_autogenerate_prefix(autogen_context),
-            "sql": str(
+            "sql": compat.text_type(
                 value.compile(dialect=autogen_context['dialect'],
                               **compile_kw)
             )
@@ -66,12 +97,12 @@ def _add_table(table, autogen_context):
         args = ',\n'.join(args)
 
     text = "%(prefix)screate_table(%(tablename)r,\n%(args)s" % {
-        'tablename': table.name,
+        'tablename': _ident(table.name),
         'prefix': _alembic_autogenerate_prefix(autogen_context),
         'args': args,
     }
     if table.schema:
-        text += ",\nschema=%r" % table.schema
+        text += ",\nschema=%r" % _ident(table.schema)
     for k in sorted(table.kwargs):
         text += ",\n%s=%r" % (k.replace(" ", "_"), table.kwargs[k])
     text += "\n)"
@@ -81,38 +112,57 @@ def _add_table(table, autogen_context):
 def _drop_table(table, autogen_context):
     text = "%(prefix)sdrop_table(%(tname)r" % {
         "prefix": _alembic_autogenerate_prefix(autogen_context),
-        "tname": table.name
+        "tname": _ident(table.name)
     }
     if table.schema:
-        text += ", schema=%r" % table.schema
+        text += ", schema=%r" % _ident(table.schema)
     text += ")"
     return text
 
 
+def _get_index_rendered_expressions(idx, autogen_context):
+    if compat.sqla_08:
+        return [repr(_ident(getattr(exp, "name", None)))
+                if isinstance(exp, sa_schema.Column)
+                else _render_potential_expr(exp, autogen_context)
+                for exp in idx.expressions]
+    else:
+        return [
+            repr(_ident(getattr(col, "name", None))) for col in idx.columns]
+
+
 def _add_index(index, autogen_context):
     """
     Generate Alembic operations for the CREATE INDEX of an
     :class:`~sqlalchemy.schema.Index` instance.
     """
-    from .compare import _get_index_column_names
 
-    text = "%(prefix)screate_index(%(name)r, '%(table)s', %(columns)s, "\
-        "unique=%(unique)r%(schema)s%(kwargs)s)" % {
-            'prefix': _alembic_autogenerate_prefix(autogen_context),
-            'name': _render_gen_name(autogen_context, index.name),
-            'table': index.table.name,
-            'columns': _get_index_column_names(index),
-            'unique': index.unique or False,
-            'schema': (", schema='%s'" % index.table.schema)
-            if index.table.schema else '',
-            'kwargs': (
-                ', ' +
-                ', '.join(
-                    ["%s=%s" %
-                     (key, _render_potential_expr(val, autogen_context))
-                     for key, val in index.kwargs.items()]))
-            if len(index.kwargs) else ''
-        }
+    has_batch = 'batch_prefix' in autogen_context
+
+    if has_batch:
+        tmpl = "%(prefix)screate_index(%(name)r, [%(columns)s], "\
+            "unique=%(unique)r%(kwargs)s)"
+    else:
+        tmpl = "%(prefix)screate_index(%(name)r, %(table)r, [%(columns)s], "\
+            "unique=%(unique)r%(schema)s%(kwargs)s)"
+
+    text = tmpl % {
+        'prefix': _alembic_autogenerate_prefix(autogen_context),
+        'name': _render_gen_name(autogen_context, index.name),
+        'table': _ident(index.table.name),
+        'columns': ", ".join(
+            _get_index_rendered_expressions(index, autogen_context)),
+        'unique': index.unique or False,
+        'schema': (", schema=%r" % _ident(index.table.schema))
+        if index.table.schema else '',
+        'kwargs': (
+            ', ' +
+            ', '.join(
+                ["%s=%s" %
+                 (key, _render_potential_expr(val, autogen_context))
+                 for key, val in index.kwargs.items()]))
+        if len(index.kwargs) else ''
+    }
     return text
 
 
@@ -121,14 +171,21 @@ def _drop_index(index, autogen_context):
     Generate Alembic operations for the DROP INDEX of an
     :class:`~sqlalchemy.schema.Index` instance.
     """
-    text = "%(prefix)sdrop_index(%(name)r, "\
-        "table_name='%(table_name)s'%(schema)s)" % {
-            'prefix': _alembic_autogenerate_prefix(autogen_context),
-            'name': _render_gen_name(autogen_context, index.name),
-            'table_name': index.table.name,
-            'schema': ((", schema='%s'" % index.table.schema)
-                       if index.table.schema else '')
-        }
+    has_batch = 'batch_prefix' in autogen_context
+
+    if has_batch:
+        tmpl = "%(prefix)sdrop_index(%(name)r)"
+    else:
+        tmpl = "%(prefix)sdrop_index(%(name)r, "\
+            "table_name=%(table_name)r%(schema)s)"
+
+    text = tmpl % {
+        'prefix': _alembic_autogenerate_prefix(autogen_context),
+        'name': _render_gen_name(autogen_context, index.name),
+        'table_name': _ident(index.table.name),
+        'schema': ((", schema=%r" % _ident(index.table.schema))
+                   if index.table.schema else '')
+    }
     return text
 
 
@@ -150,27 +207,33 @@ def _add_unique_constraint(constraint, autogen_context):
 
 def _uq_constraint(constraint, autogen_context, alter):
     opts = []
+
+    has_batch = 'batch_prefix' in autogen_context
+
     if constraint.deferrable:
         opts.append(("deferrable", str(constraint.deferrable)))
     if constraint.initially:
         opts.append(("initially", str(constraint.initially)))
-    if alter and constraint.table.schema:
-        opts.append(("schema", str(constraint.table.schema)))
+    if not has_batch and alter and constraint.table.schema:
+        opts.append(("schema", _ident(constraint.table.schema)))
     if not alter and constraint.name:
         opts.append(
-            ("name", _render_gen_name(autogen_context, constraint.name)))
+            ("name",
+             _render_gen_name(autogen_context, constraint.name)))
 
     if alter:
-        args = [repr(_render_gen_name(autogen_context, constraint.name)),
-                repr(constraint.table.name)]
-        args.append(repr([col.name for col in constraint.columns]))
+        args = [
+            repr(_render_gen_name(autogen_context, constraint.name))]
+        if not has_batch:
+            args += [repr(_ident(constraint.table.name))]
+        args.append(repr([_ident(col.name) for col in constraint.columns]))
         args.extend(["%s=%r" % (k, v) for k, v in opts])
         return "%(prefix)screate_unique_constraint(%(args)s)" % {
             'prefix': _alembic_autogenerate_prefix(autogen_context),
             'args': ", ".join(args)
         }
     else:
-        args = [repr(col.name) for col in constraint.columns]
+        args = [repr(_ident(col.name)) for col in constraint.columns]
         args.extend(["%s=%r" % (k, v) for k, v in opts])
         return "%(prefix)sUniqueConstraint(%(args)s)" % {
             "prefix": _sqlalchemy_autogenerate_prefix(autogen_context),
@@ -209,38 +272,55 @@ def _drop_constraint(constraint, autogen_context):
     Generate Alembic operations for the ALTER TABLE ... DROP CONSTRAINT
     of a  :class:`~sqlalchemy.schema.UniqueConstraint` instance.
     """
-    text = "%(prefix)sdrop_constraint"\
-        "(%(name)r, '%(table_name)s'%(schema)s)" % {
-            'prefix': _alembic_autogenerate_prefix(autogen_context),
-            'name': _render_gen_name(autogen_context, constraint.name),
-            'table_name': constraint.table.name,
-            'schema': (", schema='%s'" % constraint.table.schema)
-            if constraint.table.schema else '',
-        }
+    if 'batch_prefix' in autogen_context:
+        template = "%(prefix)sdrop_constraint"\
+            "(%(name)r)"
+    else:
+        template = "%(prefix)sdrop_constraint"\
+            "(%(name)r, '%(table_name)s'%(schema)s)"
+
+    text = template % {
+        'prefix': _alembic_autogenerate_prefix(autogen_context),
+        'name': _render_gen_name(autogen_context, constraint.name),
+        'table_name': _ident(constraint.table.name),
+        'schema': (", schema='%s'" % _ident(constraint.table.schema))
+        if constraint.table.schema else '',
+    }
     return text
 
 
 def _add_column(schema, tname, column, autogen_context):
-    text = "%(prefix)sadd_column(%(tname)r, %(column)s" % {
+    if 'batch_prefix' in autogen_context:
+        template = "%(prefix)sadd_column(%(column)s)"
+    else:
+        template = "%(prefix)sadd_column(%(tname)r, %(column)s"
+        if schema:
+            template += ", schema=%(schema)r"
+        template += ")"
+    text = template % {
         "prefix": _alembic_autogenerate_prefix(autogen_context),
         "tname": tname,
-        "column": _render_column(column, autogen_context)
+        "column": _render_column(column, autogen_context),
+        "schema": schema
     }
-    if schema:
-        text += ", schema=%r" % schema
-    text += ")"
     return text
 
 
 def _drop_column(schema, tname, column, autogen_context):
-    text = "%(prefix)sdrop_column(%(tname)r, %(cname)r" % {
+    if 'batch_prefix' in autogen_context:
+        template = "%(prefix)sdrop_column(%(cname)r)"
+    else:
+        template = "%(prefix)sdrop_column(%(tname)r, %(cname)r"
+        if schema:
+            template += ", schema=%(schema)r"
+        template += ")"
+
+    text = template % {
         "prefix": _alembic_autogenerate_prefix(autogen_context),
-        "tname": tname,
-        "cname": column.name
+        "tname": _ident(tname),
+        "cname": _ident(column.name),
+        "schema": _ident(schema)
     }
-    if schema:
-        text += ", schema=%r" % schema
-    text += ")"
     return text
 
 
@@ -254,7 +334,13 @@ def _modify_col(tname, cname,
                 existing_server_default=False,
                 schema=None):
     indent = " " * 11
-    text = "%(prefix)salter_column(%(tname)r, %(cname)r" % {
+
+    if 'batch_prefix' in autogen_context:
+        template = "%(prefix)salter_column(%(cname)r"
+    else:
+        template = "%(prefix)salter_column(%(tname)r, %(cname)r"
+
+    text = template % {
         'prefix': _alembic_autogenerate_prefix(
             autogen_context),
         'tname': tname,
@@ -282,16 +368,16 @@ def _modify_col(tname, cname,
             autogen_context)
         text += ",\n%sexisting_server_default=%s" % (
             indent, rendered)
-    if schema:
+    if schema and "batch_prefix" not in autogen_context:
         text += ",\n%sschema=%r" % (indent, schema)
     text += ")"
     return text
 
 
-def _user_autogenerate_prefix(autogen_context):
+def _user_autogenerate_prefix(autogen_context, target):
     prefix = autogen_context['opts']['user_module_prefix']
     if prefix is None:
-        return _sqlalchemy_autogenerate_prefix(autogen_context)
+        return "%s." % target.__module__
     else:
         return prefix
 
@@ -301,7 +387,10 @@ def _sqlalchemy_autogenerate_prefix(autogen_context):
 
 
 def _alembic_autogenerate_prefix(autogen_context):
-    return autogen_context['opts']['alembic_module_prefix'] or ''
+    if 'batch_prefix' in autogen_context:
+        return autogen_context['batch_prefix']
+    else:
+        return autogen_context['opts']['alembic_module_prefix'] or ''
 
 
 def _user_defined_render(type_, object_, autogen_context):
@@ -337,7 +426,7 @@ def _render_column(column, autogen_context):
     # TODO: for non-ascii colname, assign a "key"
     return "%(prefix)sColumn(%(name)r, %(type)s, %(kw)s)" % {
         'prefix': _sqlalchemy_autogenerate_prefix(autogen_context),
-        'name': column.name,
+        'name': _ident(column.name),
         'type': _repr_type(column.type, autogen_context),
         'kw': ", ".join(["%s=%s" % (kwname, val) for kwname, val in opts])
     }
@@ -349,19 +438,15 @@ def _render_server_default(default, autogen_context, repr_=True):
         return rendered
 
     if isinstance(default, sa_schema.DefaultClause):
-        if isinstance(default.arg, string_types):
+        if isinstance(default.arg, compat.string_types):
             default = default.arg
         else:
-            default = str(default.arg.compile(
-                dialect=autogen_context['dialect']))
-    if isinstance(default, string_types):
-        if repr_:
-            default = re.sub(r"^'|'$", "", default)
-            return repr(default)
-        else:
-            return default
-    else:
-        return None
+            return _render_potential_expr(default.arg, autogen_context)
+
+    if isinstance(default, string_types) and repr_:
+        default = repr(re.sub(r"^'|'$", "", default))
+
+    return default
 
 
 def _repr_type(type_, autogen_context):
@@ -380,7 +465,7 @@ def _repr_type(type_, autogen_context):
         prefix = _sqlalchemy_autogenerate_prefix(autogen_context)
         return "%s%r" % (prefix, type_)
     else:
-        prefix = _user_autogenerate_prefix(autogen_context)
+        prefix = _user_autogenerate_prefix(autogen_context, type_)
         return "%s%r" % (prefix, type_)
 
 
@@ -453,7 +538,7 @@ def _render_foreign_key(constraint, autogen_context):
         "[%(refcols)s], %(args)s)" % {
             "prefix": _sqlalchemy_autogenerate_prefix(autogen_context),
             "cols": ", ".join(
-                "'%s'" % f.parent.key for f in constraint.elements),
+                "%r" % f.parent.key for f in constraint.elements),
             "refcols": ", ".join(repr(_fk_colspec(f, apply_metadata_schema))
                                  for f in constraint.elements),
             "args": ", ".join(
@@ -481,18 +566,16 @@ def _render_check_constraint(constraint, autogen_context):
         opts.append(
             (
                 "name",
-                repr(_render_gen_name(autogen_context, constraint.name))
+                repr(
+                    _render_gen_name(autogen_context, constraint.name))
             )
         )
-    return "%(prefix)sCheckConstraint(%(sqltext)r%(opts)s)" % {
+    return "%(prefix)sCheckConstraint(%(sqltext)s%(opts)s)" % {
         "prefix": _sqlalchemy_autogenerate_prefix(autogen_context),
         "opts": ", " + (", ".join("%s=%s" % (k, v)
                                   for k, v in opts)) if opts else "",
-        "sqltext": str(
-                constraint.sqltext.compile(
-                    dialect=autogen_context['dialect']
-                )
-        )
+        "sqltext": _render_potential_expr(
+            constraint.sqltext, autogen_context, wrap_in_text=False)
     }
 
 _constraint_renderers = {
