@@ -8,6 +8,11 @@ from ..compat import string_types, text_type, with_metaclass
 from .. import util
 from . import base
 
+if util.sqla_08:
+    from sqlalchemy.sql.expression import TextClause
+else:
+    from sqlalchemy.sql.expression import _TextClause as TextClause
+
 
 class ImplMeta(type):
 
@@ -58,6 +63,27 @@ class DefaultImpl(with_metaclass(ImplMeta)):
         self.output_buffer.write(text_type(text + "\n\n"))
         self.output_buffer.flush()
 
+    def requires_recreate_in_batch(self, batch_op):
+        """Return True if the given :class:`.BatchOperationsImpl`
+        would need the table to be recreated and copied in order to
+        proceed.
+
+        Normally, only returns True on SQLite when operations other
+        than add_column are present.
+
+        """
+        return False
+
+    def prep_table_for_batch(self, table):
+        """perform any operations needed on a table before a new
+        one is created to replace it in batch mode.
+
+        the PG dialect uses this to drop constraints on the table
+        before the new one uses those same names.
+
+        """
+
+
     @property
     def bind(self):
         return self.connection
@@ -78,7 +104,7 @@ class DefaultImpl(with_metaclass(ImplMeta)):
             conn = self.connection
             if execution_options:
                 conn = conn.execution_options(**execution_options)
-            conn.execute(construct, *multiparams, **params)
+            return conn.execute(construct, *multiparams, **params)
 
     def execute(self, sql, execution_options=None):
         self._exec(sql, execution_options)
@@ -226,6 +252,24 @@ class DefaultImpl(with_metaclass(ImplMeta)):
                                         metadata_indexes):
         pass
 
+    def _compat_autogen_column_reflect(self, inspector):
+        if util.sqla_08:
+            return self.autogen_column_reflect
+        else:
+            def adapt(table, column_info):
+                return self.autogen_column_reflect(
+                    inspector, table, column_info)
+            return adapt
+
+    def autogen_column_reflect(self, inspector, table, column_info):
+        """A hook that is attached to the 'column_reflect' event for when
+        a Table is reflected from the database during the autogenerate
+        process.
+
+        Dialects can elect to modify the information gathered here.
+
+        """
+
     def start_migrations(self):
         """A hook called when :meth:`.EnvironmentContext.run_migrations`
         is called.
@@ -270,14 +314,13 @@ def _textual_index_column(table, text_):
         c = schema.Column(text_, sqltypes.NULLTYPE)
         table.append_column(c)
         return c
-    elif isinstance(text_, expression.TextClause):
+    elif isinstance(text_, TextClause):
         return _textual_index_element(table, text_)
     else:
         raise ValueError("String or text() construct expected")
 
 
 class _textual_index_element(sql.ColumnElement):
-
     """Wrap around a sqlalchemy text() construct in such a way that
     we appear like a column-oriented SQL expression to an Index
     construct.
