@@ -72,10 +72,30 @@ class MySQLImpl(DefaultImpl):
                 )
             )
 
+    def compare_server_default(self, inspector_column,
+                               metadata_column,
+                               rendered_metadata_default,
+                               rendered_inspector_default):
+        # partially a workaround for SQLAlchemy issue #3023; if the
+        # column were created without "NOT NULL", MySQL may have added
+        # an implicit default of '0' which we need to skip
+        if metadata_column.type._type_affinity is sqltypes.Integer and \
+            inspector_column.primary_key and \
+                not inspector_column.autoincrement and \
+                not rendered_metadata_default and \
+                rendered_inspector_default == "'0'":
+            return False
+        else:
+            return rendered_inspector_default != rendered_metadata_default
+
     def correct_for_autogen_constraints(self, conn_unique_constraints,
                                         conn_indexes,
                                         metadata_unique_constraints,
                                         metadata_indexes):
+
+        # TODO: if SQLA 1.0, make use of "duplicates_index"
+        # metadata
+
         removed = set()
         for idx in list(conn_indexes):
             # MySQL puts implicit indexes on FK columns, even if
@@ -92,6 +112,8 @@ class MySQLImpl(DefaultImpl):
                         conn_indexes.remove(idx)
                         removed.add(idx.name)
                         break
+                if idx.name in removed:
+                    break
 
         # then remove indexes from the "metadata_indexes"
         # that we've removed from reflected, otherwise they come out
