@@ -11,6 +11,21 @@ class SQLiteImpl(DefaultImpl):
     see: http://bugs.python.org/issue10740
     """
 
+    def requires_recreate_in_batch(self, batch_op):
+        """Return True if the given :class:`.BatchOperationsImpl`
+        would need the table to be recreated and copied in order to
+        proceed.
+
+        Normally, only returns True on SQLite when operations other
+        than add_column are present.
+
+        """
+        for op in batch_op.batch:
+            if op[0] != 'add_column':
+                return True
+        else:
+            return False
+
     def add_constraint(self, const):
         # attempt to distinguish between an
         # auto-gen constraint and an explicit one
@@ -31,9 +46,14 @@ class SQLiteImpl(DefaultImpl):
                                rendered_metadata_default,
                                rendered_inspector_default):
 
-        rendered_metadata_default = re.sub(
-            r"^'|'$", "", rendered_metadata_default)
-        return rendered_inspector_default != repr(rendered_metadata_default)
+        if rendered_metadata_default is not None:
+            rendered_metadata_default = re.sub(
+                r"^\"'|\"'$", "", rendered_metadata_default)
+        if rendered_inspector_default is not None:
+            rendered_inspector_default = re.sub(
+                r"^\"'|\"'$", "", rendered_inspector_default)
+
+        return rendered_inspector_default != rendered_metadata_default
 
     def correct_for_autogen_constraints(
         self, conn_unique_constraints, conn_indexes,
