@@ -2,7 +2,9 @@ from .operations import Operations
 from .migration import MigrationContext
 from . import util
 
+
 class EnvironmentContext(object):
+
     """Represent the state made available to an ``env.py`` script.
 
     :class:`.EnvironmentContext` is normally instantiated
@@ -134,13 +136,33 @@ class EnvironmentContext(object):
         return not self.is_offline_mode()
 
     def get_head_revision(self):
-        """Return the hex identifier of the 'head' revision.
+        """Return the hex identifier of the 'head' script revision.
+
+        If the script directory has multiple heads, this
+        method raises a :class:`.CommandError`;
+        :meth:`.EnvironmentContext.get_head_revisions` should be preferred.
 
         This function does not require that the :class:`.MigrationContext`
         has been configured.
 
+        .. seealso:: :meth:`.EnvironmentContext.get_head_revisions`
+
         """
-        return self.script._as_rev_number("head")
+        return self.script.as_revision_number("head")
+
+    def get_head_revisions(self):
+        """Return the hex identifier of the 'heads' script revision(s).
+
+        This returns a tuple containing the version number of all
+        heads in the script directory.
+
+        This function does not require that the :class:`.MigrationContext`
+        has been configured.
+
+        .. versionadded:: 0.7.0
+
+        """
+        return self.script.as_revision_number("heads")
 
     def get_starting_revision_argument(self):
         """Return the 'starting revision' argument,
@@ -155,14 +177,18 @@ class EnvironmentContext(object):
 
         """
         if self._migration_context is not None:
-            return self.script._as_rev_number(
-                        self.get_context()._start_from_rev)
+            return self.script.as_revision_number(
+                self.get_context()._start_from_rev)
         elif 'starting_rev' in self.context_opts:
-            return self.script._as_rev_number(
-                        self.context_opts['starting_rev'])
+            return self.script.as_revision_number(
+                self.context_opts['starting_rev'])
         else:
+            # this should raise only in the case that a command
+            # is being run where the "starting rev" is never applicable;
+            # this is to catch scripts which rely upon this in
+            # non-sql mode or similar
             raise util.CommandError(
-                        "No starting revision argument is available.")
+                "No starting revision argument is available.")
 
     def get_revision_argument(self):
         """Get the 'destination' revision argument.
@@ -178,8 +204,8 @@ class EnvironmentContext(object):
         has been configured.
 
         """
-        return self.script._as_rev_number(
-                            self.context_opts['destination_rev'])
+        return self.script.as_revision_number(
+            self.context_opts['destination_rev'])
 
     def get_tag_argument(self):
         """Return the value passed for the ``--tag`` argument, if any.
@@ -216,7 +242,8 @@ class EnvironmentContext(object):
         For example, to support passing a database URL on the command line,
         the standard ``env.py`` script can be modified like this::
 
-            cmd_line_url = context.get_x_argument(as_dictionary=True).get('dbname')
+            cmd_line_url = context.get_x_argument(
+                as_dictionary=True).get('dbname')
             if cmd_line_url:
                 engine = create_engine(cmd_line_url)
             else:
@@ -247,34 +274,35 @@ class EnvironmentContext(object):
             value = []
         if as_dictionary:
             value = dict(
-                        arg.split('=', 1) for arg in value
-                    )
+                arg.split('=', 1) for arg in value
+            )
         return value
 
     def configure(self,
-            connection=None,
-            url=None,
-            dialect_name=None,
-            transactional_ddl=None,
-            transaction_per_migration=False,
-            output_buffer=None,
-            starting_rev=None,
-            tag=None,
-            template_args=None,
-            target_metadata=None,
-            include_symbol=None,
-            include_object=None,
-            include_schemas=False,
-            compare_type=False,
-            compare_server_default=False,
-            render_item=None,
-            upgrade_token="upgrades",
-            downgrade_token="downgrades",
-            alembic_module_prefix="op.",
-            sqlalchemy_module_prefix="sa.",
-            user_module_prefix=None,
-            **kw
-        ):
+                  connection=None,
+                  url=None,
+                  dialect_name=None,
+                  transactional_ddl=None,
+                  transaction_per_migration=False,
+                  output_buffer=None,
+                  starting_rev=None,
+                  tag=None,
+                  template_args=None,
+                  render_as_batch=False,
+                  target_metadata=None,
+                  include_symbol=None,
+                  include_object=None,
+                  include_schemas=False,
+                  compare_type=False,
+                  compare_server_default=False,
+                  render_item=None,
+                  upgrade_token="upgrades",
+                  downgrade_token="downgrades",
+                  alembic_module_prefix="op.",
+                  sqlalchemy_module_prefix="sa.",
+                  user_module_prefix=None,
+                  **kw
+                  ):
         """Configure a :class:`.MigrationContext` within this
         :class:`.EnvironmentContext` which will provide database
         connectivity and other configuration to a series of
@@ -338,8 +366,6 @@ class EnvironmentContext(object):
         :param output_encoding: when using ``--sql`` to generate SQL
          scripts, apply this encoding to the string output.
 
-         .. versionadded:: 0.5.0
-
         :param starting_rev: Override the "starting revision" argument
          when using ``--sql`` mode.
         :param tag: a string tag for usage by custom ``env.py`` scripts.
@@ -351,15 +377,11 @@ class EnvironmentContext(object):
          option is used, or if the option "revision_environment=true"
          is present in the alembic.ini file.
 
-         .. versionadded:: 0.3.3
-
         :param version_table: The name of the Alembic version table.
          The default is ``'alembic_version'``.
         :param version_table_schema: Optional schema to place version
          table within.
 
-         .. versionadded:: 0.5.0
-
         Parameters specific to the autogenerate feature, when
         ``alembic revision`` is run with the ``--autogenerate`` feature:
 
@@ -396,10 +418,12 @@ class EnvironmentContext(object):
             )
 
 
-         ``inspected_column`` is a :class:`sqlalchemy.schema.Column` as returned by
-         :meth:`sqlalchemy.engine.reflection.Inspector.reflecttable`, whereas
-         ``metadata_column`` is a :class:`sqlalchemy.schema.Column` from
-         the local model environment.
+         ``inspected_column`` is a :class:`sqlalchemy.schema.Column` as
+         returned by
+         :meth:`sqlalchemy.engine.reflection.Inspector.reflecttable`,
+         whereas ``metadata_column`` is a
+         :class:`sqlalchemy.schema.Column` from the local model
+         environment.
 
          A return value of ``None`` indicates to allow default type
          comparison to proceed.
@@ -458,13 +482,20 @@ class EnvironmentContext(object):
 
          The function accepts the following positional arguments:
 
-         * ``object``: a :class:`~sqlalchemy.schema.SchemaItem` object such as a
-           :class:`~sqlalchemy.schema.Table` or :class:`~sqlalchemy.schema.Column`
-           object
+         * ``object``: a :class:`~sqlalchemy.schema.SchemaItem` object such
+           as a :class:`~sqlalchemy.schema.Table`,
+           :class:`~sqlalchemy.schema.Column`,
+           :class:`~sqlalchemy.schema.Index`
+           or :class:`~sqlalchemy.schema.UniqueConstraint` object
          * ``name``: the name of the object. This is typically available
            via ``object.name``.
          * ``type``: a string describing the type of object; currently
-           ``"table"`` or ``"column"``
+           ``"table"``, ``"column"``, ``"index"`` or ``"unique_constraint"``.
+
+           .. versionadded:: 0.7.0 Support for indexes and unique constraints
+              within the
+              :paramref:`~.EnvironmentContext.configure.include_object` hook.
+
          * ``reflected``: ``True`` if the given object was produced based on
            table reflection, ``False`` if it's from a local :class:`.MetaData`
            object.
@@ -500,10 +531,12 @@ class EnvironmentContext(object):
             :paramref:`.EnvironmentContext.configure.include_schemas`
 
         :param include_symbol: A callable function which, given a table name
-         and schema name (may be ``None``), returns ``True`` or ``False``, indicating
-         if the given table should be considered in the autogenerate sweep.
+         and schema name (may be ``None``), returns ``True`` or ``False``,
+         indicating if the given table should be considered in the
+         autogenerate sweep.
 
-         .. deprecated:: 0.6.0 :paramref:`.EnvironmentContext.configure.include_symbol`
+         .. deprecated:: 0.6.0
+            :paramref:`.EnvironmentContext.configure.include_symbol`
             is superceded by the more generic
             :paramref:`.EnvironmentContext.configure.include_object`
             parameter.
@@ -524,6 +557,16 @@ class EnvironmentContext(object):
 
             :paramref:`.EnvironmentContext.configure.include_object`
 
+        :param render_as_batch: if True, commands which alter elements
+         within a table will be placed under a ``with batch_alter_table():``
+         directive, so that batch migrations will take place.
+
+         .. versionadded:: 0.7.0
+
+         .. seealso::
+
+            :ref:`batch_migrations`
+
         :param include_schemas: If True, autogenerate will scan across
          all schemas located by the SQLAlchemy
          :meth:`~sqlalchemy.engine.reflection.Inspector.get_schema_names`
@@ -533,8 +576,6 @@ class EnvironmentContext(object):
          option to specify a callable which
          can filter the tables/schemas that get included.
 
-         .. versionadded :: 0.4.0
-
          .. seealso::
 
             :paramref:`.EnvironmentContext.configure.include_object`
@@ -564,8 +605,6 @@ class EnvironmentContext(object):
          ``"primary_key"``, ``"foreign_key"``, ``"unique"``, ``"check"``,
          ``"type"``, ``"server_default"``.
 
-         .. versionadded:: 0.5.0
-
          .. seealso::
 
             :ref:`autogen_render_types`
@@ -597,10 +636,18 @@ class EnvironmentContext(object):
         :param user_module_prefix: When autogenerate refers to a SQLAlchemy
          type (e.g. :class:`.TypeEngine`) where the module name is not
          under the ``sqlalchemy`` namespace, this prefix will be used
-         within autogenerate, if non-``None``; if left at its default of
-         ``None``, the
-         :paramref:`.EnvironmentContext.configure.sqlalchemy_module_prefix`
-         is used instead.
+         within autogenerate.  If left at its default of
+         ``None``, the ``__module__`` attribute of the type is used to
+         render the import module.   It's a good practice to set this
+         and to have all custom types be available from a fixed module space,
+         in order to future-proof migration files against reorganizations
+         in modules.
+
+         .. versionchanged:: 0.7.0
+            :paramref:`.EnvironmentContext.configure.user_module_prefix`
+            no longer defaults to the value of
+            :paramref:`.EnvironmentContext.configure.sqlalchemy_module_prefix`
+            when left at ``None``; the ``__module__`` attribute is now used.
 
          .. versionadded:: 0.6.3 added
             :paramref:`.EnvironmentContext.configure.user_module_prefix`
@@ -642,6 +689,7 @@ class EnvironmentContext(object):
         opts['include_symbol'] = include_symbol
         opts['include_object'] = include_object
         opts['include_schemas'] = include_schemas
+        opts['render_as_batch'] = True #render_as_batch
         opts['upgrade_token'] = upgrade_token
         opts['downgrade_token'] = downgrade_token
         opts['sqlalchemy_module_prefix'] = sqlalchemy_module_prefix
@@ -701,7 +749,7 @@ class EnvironmentContext(object):
 
         """
         self.get_context().execute(sql,
-                execution_options=execution_options)
+                                   execution_options=execution_options)
 
     def static_output(self, text):
         """Emit text directly to the "offline" SQL stream.
@@ -714,7 +762,6 @@ class EnvironmentContext(object):
         """
         self.get_context().impl.static_output(text)
 
-
     def begin_transaction(self):
         """Return a context manager that will
         enclose an operation within a "transaction",
@@ -761,7 +808,6 @@ class EnvironmentContext(object):
 
         return self.get_context().begin_transaction()
 
-
     def get_context(self):
         """Return the current :class:`.MigrationContext` object.
 
@@ -789,4 +835,3 @@ class EnvironmentContext(object):
 
     def get_impl(self):
         return self.get_context().impl
-
