@@ -3,11 +3,11 @@ from contextlib import contextmanager
 from sqlalchemy.types import NULLTYPE, Integer
 from sqlalchemy import schema as sa_schema
 
-from . import util
+from . import util, batch
 from .compat import string_types
 from .ddl import impl
 
-__all__ = ('Operations',)
+__all__ = ('Operations', 'BatchOperations')
 
 try:
     from sqlalchemy.sql.naming import conv
@@ -42,7 +42,7 @@ class Operations(object):
 
     """
 
-    def __init__(self, migration_context):
+    def __init__(self, migration_context, impl=None):
         """Construct a new :class:`.Operations`
 
         :param migration_context: a :class:`.MigrationContext`
@@ -50,7 +50,10 @@ class Operations(object):
 
         """
         self.migration_context = migration_context
-        self.impl = migration_context.impl
+        if impl is None:
+            self.impl = migration_context.impl
+        else:
+            self.impl = impl
 
     @classmethod
     @contextmanager
@@ -94,6 +97,11 @@ class Operations(object):
 
         tname = "%s.%s" % (referent_schema, referent) if referent_schema \
                 else referent
+
+        if util.sqla_08:
+            # "match" kw unsupported in 0.7
+            dialect_kw['match'] = match
+
         f = sa_schema.ForeignKeyConstraint(local_cols,
                                            ["%s.%s" % (tname, n)
                                             for n in remote_cols],
@@ -102,7 +110,6 @@ class Operations(object):
                                            ondelete=ondelete,
                                            deferrable=deferrable,
                                            initially=initially,
-                                           match=match,
                                            **dialect_kw
                                            )
         t1.append_constraint(f)
@@ -182,6 +189,95 @@ class Operations(object):
             if cname not in rel_t.c:
                 rel_t.append_column(sa_schema.Column(cname, NULLTYPE))
 
+    @contextmanager
+    def batch_alter_table(
+            self, table_name, schema=None, recreate="auto", copy_from=None,
+            table_args=(), table_kwargs=util.immutabledict()):
+        """Invoke a series of per-table migrations in batch.
+
+        Batch mode allows a series of operations specific to a table
+        to be syntactically grouped together, and allows for alternate
+        modes of table migration, in particular the "recreate" style of
+        migration required by SQLite.
+
+        "recreate" style is as follows:
+
+        1. A new table is created with the new specification, based on the
+           migration directives within the batch, using a temporary name.
+
+        2. the data copied from the existing table to the new table.
+
+        3. the existing table is dropped.
+
+        4. the new table is renamed to the existing table name.
+
+        The directive by default will only use "recreate" style on the
+        SQLite backend, and only if directives are present which require
+        this form, e.g. anything other than ``add_column()``.   The batch
+        operation on other backends will proceed using standard ALTER TABLE
+        operations.
+
+        The method is used as a context manager, which returns an instance
+        of :class:`.BatchOperations`; this object is the same as
+        :class:`.Operations` except that table names and schema names
+        are omitted.  E.g.::
+
+            with op.batch_alter_table("some_table") as batch_op:
+                batch_op.add_column(Column('foo', Integer))
+                batch_op.drop_column('bar')
+
+        The operations within the context manager are invoked at once
+        when the context is ended.   When run against SQLite, if the
+        migrations include operations not supported by SQLite's ALTER TABLE,
+        the entire table will be copied to a new one with the new
+        specification, moving all data across as well.
+
+        The copy operation by default uses reflection to retrieve the current
+        structure of the table, and therefore :meth:`.batch_alter_table`
+        in this mode requires that the migration is run in "online" mode.
+        The ``copy_from`` parameter may be passed which refers to an existing
+        :class:`.Table` object, which will bypass this reflection step.
+
+        .. note::  The table copy operation will currently not copy
+           CHECK constraints, and may not copy UNIQUE constraints that are
+           unnamed, as is possible on SQLite.
+
+        :param table_name: name of table
+        :param schema: optional schema name.
+        :param recreate: under what circumstances the table should be
+         recreated. At its default of ``"auto"``, the SQLite dialect will
+         recreate the table if any operations other than ``add_column()`` are
+         present. Other options include ``"always"`` and ``"never"``.
+        :param copy_from: optional :class:`~sqlalchemy.schema.Table` object
+         that will act as the structure of the table being copied.  If omitted,
+         table reflection is used to retrieve the structure of the table.
+        :param table_args: a sequence of additional positional arguments that
+         will be applied to the new :class:`~sqlalchemy.schema.Table` when
+         created, in addition to those copied from the source table.
+         This may be used to provide additional constraints such as CHECK
+         constraints that may not be reflected.
+        :param table_kwargs: a dictionary of additional keyword arguments
+         that will be applied to the new :class:`~sqlalchemy.schema.Table`
+         when created, in addition to those copied from the source table.
+         This may be used to provide for additional table options that may
+         not be reflected.
+
+        .. versionadded:: 0.7.0
+
+        .. note:: batch mode requires SQLAlchemy 0.8 or above.
+
+        .. seealso::
+
+            :ref:`batch_migrations`
+
+        """
+        impl = batch.BatchOperationsImpl(
+            self, table_name, schema, recreate,
+            copy_from, table_args, table_kwargs)
+        batch_op = BatchOperations(self.migration_context, impl=impl)
+        yield batch_op
+        impl.flush()
+
     def get_context(self):
         """Return the :class:`.MigrationContext` object that's
         currently in use.
@@ -195,7 +291,13 @@ class Operations(object):
 
         :param old_table_name: old name.
         :param new_table_name: new name.
-        :param schema: Optional schema name to operate within.
+        :param schema: Optional schema name to operate within.  To control
+         quoting of the schema outside of the default behavior, use
+         the SQLAlchemy construct
+         :class:`~sqlalchemy.sql.elements.quoted_name`.
+
+         .. versionadded:: 0.7.0 'schema' can now accept a
+            :class:`~sqlalchemy.sql.elements.quoted_name` construct.
 
         """
         self.impl.rename_table(
@@ -255,12 +357,6 @@ class Operations(object):
          Set to ``None`` to have the default removed.
         :param new_column_name: Optional; specify a string name here to
          indicate the new name within a column rename operation.
-
-         .. versionchanged:: 0.5.0
-            The ``name`` parameter is now named ``new_column_name``.
-            The old name will continue to function for backwards
-            compatibility.
-
         :param ``type_``: Optional; a :class:`~sqlalchemy.types.TypeEngine`
          type object to specify a change to the column's type.
          For SQLAlchemy types that also indicate a constraint (i.e.
@@ -289,9 +385,13 @@ class Operations(object):
         :param existing_autoincrement: Optional; the existing autoincrement
          of the column.  Used for MySQL's system of altering a column
          that specifies ``AUTO_INCREMENT``.
-        :param schema: Optional schema name to operate within.
+        :param schema: Optional schema name to operate within.  To control
+         quoting of the schema outside of the default behavior, use
+         the SQLAlchemy construct
+         :class:`~sqlalchemy.sql.elements.quoted_name`.
 
-         .. versionadded:: 0.4.0
+         .. versionadded:: 0.7.0 'schema' can now accept a
+            :class:`~sqlalchemy.sql.elements.quoted_name` construct.
 
         """
 
@@ -428,9 +528,14 @@ class Operations(object):
         :param table_name: String name of the parent table.
         :param column: a :class:`sqlalchemy.schema.Column` object
          representing the new column.
-        :param schema: Optional schema name to operate within.
+        :param schema: Optional schema name to operate within.  To control
+         quoting of the schema outside of the default behavior, use
+         the SQLAlchemy construct
+         :class:`~sqlalchemy.sql.elements.quoted_name`.
+
+         .. versionadded:: 0.7.0 'schema' can now accept a
+            :class:`~sqlalchemy.sql.elements.quoted_name` construct.
 
-         .. versionadded:: 0.4.0
 
         """
 
@@ -443,6 +548,8 @@ class Operations(object):
         for constraint in t.constraints:
             if not isinstance(constraint, sa_schema.PrimaryKeyConstraint):
                 self.impl.add_constraint(constraint)
+        for index in t.indexes:
+            self.impl._exec(sa_schema.CreateIndex(index))
 
     def drop_column(self, table_name, column_name, **kw):
         """Issue a "drop column" instruction using the current
@@ -454,9 +561,13 @@ class Operations(object):
 
         :param table_name: name of table
         :param column_name: name of column
-        :param schema: Optional schema name to operate within.
+        :param schema: Optional schema name to operate within.  To control
+         quoting of the schema outside of the default behavior, use
+         the SQLAlchemy construct
+         :class:`~sqlalchemy.sql.elements.quoted_name`.
 
-         .. versionadded:: 0.4.0
+         .. versionadded:: 0.7.0 'schema' can now accept a
+            :class:`~sqlalchemy.sql.elements.quoted_name` construct.
 
         :param mssql_drop_check: Optional boolean.  When ``True``, on
          Microsoft SQL Server only, first
@@ -511,8 +622,6 @@ class Operations(object):
         off normally.   The :class:`~sqlalchemy.schema.AddConstraint`
         construct is ultimately used to generate the ALTER statement.
 
-        .. versionadded:: 0.5.0
-
         :param name: Name of the primary key constraint.  The name is necessary
          so that an ALTER statement can be emitted.  For setups that
          use an automated naming scheme such as that described at
@@ -523,7 +632,13 @@ class Operations(object):
         :param table_name: String name of the target table.
         :param cols: a list of string column names to be applied to the
          primary key constraint.
-        :param schema: Optional schema name of the table.
+        :param schema: Optional schema name to operate within.  To control
+         quoting of the schema outside of the default behavior, use
+         the SQLAlchemy construct
+         :class:`~sqlalchemy.sql.elements.quoted_name`.
+
+         .. versionadded:: 0.7.0 'schema' can now accept a
+            :class:`~sqlalchemy.sql.elements.quoted_name` construct.
 
         """
         self.impl.add_constraint(
@@ -626,9 +741,13 @@ class Operations(object):
          NOT DEFERRABLE when issuing DDL for this constraint.
         :param initially: optional string. If set, emit INITIALLY <value>
          when issuing DDL for this constraint.
-        :param schema: Optional schema name to operate within.
+        :param schema: Optional schema name to operate within.  To control
+         quoting of the schema outside of the default behavior, use
+         the SQLAlchemy construct
+         :class:`~sqlalchemy.sql.elements.quoted_name`.
 
-         .. versionadded:: 0.4.0
+         .. versionadded:: 0.7.0 'schema' can now accept a
+            :class:`~sqlalchemy.sql.elements.quoted_name` construct.
 
         """
 
@@ -673,9 +792,13 @@ class Operations(object):
          NOT DEFERRABLE when issuing DDL for this constraint.
         :param initially: optional string. If set, emit INITIALLY <value>
          when issuing DDL for this constraint.
-        :param schema: Optional schema name to operate within.
+        :param schema: Optional schema name to operate within.  To control
+         quoting of the schema outside of the default behavior, use
+         the SQLAlchemy construct
+         :class:`~sqlalchemy.sql.elements.quoted_name`.
 
-         ..versionadded:: 0.4.0
+         .. versionadded:: 0.7.0 'schema' can now accept a
+            :class:`~sqlalchemy.sql.elements.quoted_name` construct.
 
         """
         self.impl.add_constraint(
@@ -698,7 +821,7 @@ class Operations(object):
                 'account',
                 Column('id', INTEGER, primary_key=True),
                 Column('name', VARCHAR(50), nullable=False),
-                Column('description', NVARCHAR(200))
+                Column('description', NVARCHAR(200)),
                 Column('timestamp', TIMESTAMP, server_default=func.now())
             )
 
@@ -718,20 +841,59 @@ class Operations(object):
                 Column('timestamp', TIMESTAMP, server_default=func.now())
             )
 
+        The function also returns a newly created
+        :class:`~sqlalchemy.schema.Table` object, corresponding to the table
+        specification given, which is suitable for
+        immediate SQL operations, in particular
+        :meth:`.Operations.bulk_insert`::
+
+            from sqlalchemy import INTEGER, VARCHAR, NVARCHAR, Column
+            from alembic import op
+
+            account_table = op.create_table(
+                'account',
+                Column('id', INTEGER, primary_key=True),
+                Column('name', VARCHAR(50), nullable=False),
+                Column('description', NVARCHAR(200)),
+                Column('timestamp', TIMESTAMP, server_default=func.now())
+            )
+
+            op.bulk_insert(
+                account_table,
+                [
+                    {"name": "A1", "description": "account 1"},
+                    {"name": "A2", "description": "account 2"},
+                ]
+            )
+
+        .. versionadded:: 0.7.0
+
         :param name: Name of the table
         :param \*columns: collection of :class:`~sqlalchemy.schema.Column`
          objects within
          the table, as well as optional :class:`~sqlalchemy.schema.Constraint`
          objects
          and :class:`~.sqlalchemy.schema.Index` objects.
-        :param schema: Optional schema name to operate within.
+        :param schema: Optional schema name to operate within.  To control
+         quoting of the schema outside of the default behavior, use
+         the SQLAlchemy construct
+         :class:`~sqlalchemy.sql.elements.quoted_name`.
+
+         .. versionadded:: 0.7.0 'schema' can now accept a
+            :class:`~sqlalchemy.sql.elements.quoted_name` construct.
         :param \**kw: Other keyword arguments are passed to the underlying
          :class:`sqlalchemy.schema.Table` object created for the command.
 
+        :return: the :class:`~sqlalchemy.schema.Table` object corresponding
+         to the parameters given.
+
+         .. versionadded:: 0.7.0 - the :class:`~sqlalchemy.schema.Table`
+            object is returned.
+
         """
-        self.impl.create_table(
-            self._table(name, *columns, **kw)
-        )
+        table = self._table(name, *columns, **kw)
+        self.impl.create_table(table)
+        return table
 
     def drop_table(self, name, **kw):
         """Issue a "drop table" instruction using the current
@@ -743,9 +905,13 @@ class Operations(object):
             drop_table("accounts")
 
         :param name: Name of the table
-        :param schema: Optional schema name to operate within.
+        :param schema: Optional schema name to operate within.  To control
+         quoting of the schema outside of the default behavior, use
+         the SQLAlchemy construct
+         :class:`~sqlalchemy.sql.elements.quoted_name`.
 
-         .. versionadded:: 0.4.0
+         .. versionadded:: 0.7.0 'schema' can now accept a
+            :class:`~sqlalchemy.sql.elements.quoted_name` construct.
 
         :param \**kw: Other keyword arguments are passed to the underlying
          :class:`sqlalchemy.schema.Table` object created for the command.
@@ -755,7 +921,8 @@ class Operations(object):
             self._table(name, **kw)
         )
 
-    def create_index(self, name, table_name, columns, schema=None, **kw):
+    def create_index(self, name, table_name, columns, schema=None,
+                     unique=False, quote=None, **kw):
         """Issue a "create index" instruction using the current
         migration context.
 
@@ -779,22 +946,36 @@ class Operations(object):
 
         :param name: name of the index.
         :param table_name: name of the owning table.
-
-         .. versionchanged:: 0.5.0
-            The ``tablename`` parameter is now named ``table_name``.
-            As this is a positional argument, the old name is no
-            longer present.
-
         :param columns: a list consisting of string column names and/or
          :func:`~sqlalchemy.sql.expression.text` constructs.
-        :param schema: Optional schema name to operate within.
+        :param schema: Optional schema name to operate within.  To control
+         quoting of the schema outside of the default behavior, use
+         the SQLAlchemy construct
+         :class:`~sqlalchemy.sql.elements.quoted_name`.
 
-         .. versionadded:: 0.4.0
+         .. versionadded:: 0.7.0 'schema' can now accept a
+            :class:`~sqlalchemy.sql.elements.quoted_name` construct.
 
+        :param unique: If True, create a unique index.
+
+        :param quote:
+            Force quoting of this column's name on or off, corresponding
+            to ``True`` or ``False``. When left at its default
+            of ``None``, the column identifier will be quoted according to
+            whether the name is case sensitive (identifiers with at least one
+            upper case character are treated as case sensitive), or if it's a
+            reserved word. This flag is only needed to force quoting of a
+            reserved word which is not known by the SQLAlchemy dialect.
+
+        :param \**kw: Additional keyword arguments not mentioned above are
+            dialect specific, and passed in the form ``<dialectname>_<argname>``.
+            See the documentation regarding an individual dialect at
+            :ref:`dialect_toplevel` for detail on documented arguments.
         """
 
         self.impl.create_index(
-            self._index(name, table_name, columns, schema=schema, **kw)
+            self._index(name, table_name, columns, schema=schema,
+                        unique=unique, quote=quote, **kw)
         )
 
     @util._with_legacy_names([('tablename', 'table_name')])
@@ -809,15 +990,13 @@ class Operations(object):
         :param name: name of the index.
         :param table_name: name of the owning table.  Some
          backends such as Microsoft SQL Server require this.
+        :param schema: Optional schema name to operate within.  To control
+         quoting of the schema outside of the default behavior, use
+         the SQLAlchemy construct
+         :class:`~sqlalchemy.sql.elements.quoted_name`.
 
-         .. versionchanged:: 0.5.0
-            The ``tablename`` parameter is now named ``table_name``.
-            The old name will continue to function for backwards
-            compatibility.
-
-        :param schema: Optional schema name to operate within.
-
-         .. versionadded:: 0.4.0
+         .. versionadded:: 0.7.0 'schema' can now accept a
+            :class:`~sqlalchemy.sql.elements.quoted_name` construct.
 
         """
         # need a dummy column name here since SQLAlchemy
@@ -832,25 +1011,15 @@ class Operations(object):
 
         :param name: name of the constraint.
         :param table_name: table name.
-
-         .. versionchanged:: 0.5.0
-            The ``tablename`` parameter is now named ``table_name``.
-            As this is a positional argument, the old name is no
-            longer present.
-
         :param ``type_``: optional, required on MySQL.  can be
          'foreignkey', 'primary', 'unique', or 'check'.
+        :param schema: Optional schema name to operate within.  To control
+         quoting of the schema outside of the default behavior, use
+         the SQLAlchemy construct
+         :class:`~sqlalchemy.sql.elements.quoted_name`.
 
-         .. versionchanged:: 0.5.0
-            The ``type`` parameter is now named ``type_``.  The old name
-            ``type`` will remain for backwards compatibility.
-
-         .. versionadded:: 0.3.6 'primary' qualfier to enable
-            dropping of MySQL primary key constraints.
-
-        :param schema: Optional schema name to operate within.
-
-         .. versionadded:: 0.4.0
+         .. versionadded:: 0.7.0 'schema' can now accept a
+            :class:`~sqlalchemy.sql.elements.quoted_name` construct.
 
         """
 
@@ -1070,3 +1239,162 @@ class Operations(object):
 
         """
         return self.migration_context.impl.bind
+
+
+class BatchOperations(Operations):
+    """Modifies the interface :class:`.Operations` for batch mode.
+
+    This basically omits the ``table_name`` and ``schema`` parameters
+    from associated methods, as these are a given when running under batch
+    mode.
+
+    .. seealso::
+
+        :meth:`.Operations.batch_alter_table`
+
+    """
+
+    def _noop(self, operation):
+        raise NotImplementedError(
+            "The %s method does not apply to a batch table alter operation."
+            % operation)
+
+    def add_column(self, column):
+        """Issue an "add column" instruction using the current
+        batch migration context.
+
+        .. seealso::
+
+            :meth:`.Operations.add_column`
+
+        """
+
+        return super(BatchOperations, self).add_column(
+            self.impl.table_name, column, schema=self.impl.schema)
+
+    def alter_column(self, column_name, **kw):
+        """Issue an "alter column" instruction using the current
+        batch migration context.
+
+        .. seealso::
+
+            :meth:`.Operations.add_column`
+
+        """
+        kw['schema'] = self.impl.schema
+        return super(BatchOperations, self).alter_column(
+            self.impl.table_name, column_name, **kw)
+
+    def drop_column(self, column_name):
+        """Issue a "drop column" instruction using the current
+        batch migration context.
+
+        .. seealso::
+
+            :meth:`.Operations.drop_column`
+
+        """
+        return super(BatchOperations, self).drop_column(
+            self.impl.table_name, column_name, schema=self.impl.schema)
+
+    def create_primary_key(self, name, cols):
+        """Issue a "create priamry key" instruction using the
+        current batch migration context.
+
+        The batch form of this call omits the ``table_name`` and ``schema``
+        arguments from the call.
+
+        .. seealso::
+
+            :meth:`.Operations.create_primary_key`
+
+        """
+        raise NotImplementedError("not yet implemented")
+
+    def create_foreign_key(
+            self, name, referent, local_cols, remote_cols, **kw):
+        """Issue a "create foreign key" instruction using the
+        current batch migration context.
+
+        The batch form of this call omits the ``source`` and ``source_schema``
+        arguments from the call.
+
+        e.g.::
+
+            with batch_alter_table("address") as batch_op:
+                batch_op.create_foreign_key(
+                            "fk_user_address",
+                            "user", ["user_id"], ["id"])
+
+        .. seealso::
+
+            :meth:`.Operations.create_foreign_key`
+
+        """
+        return super(BatchOperations, self).create_foreign_key(
+            name, self.impl.table_name, referent, local_cols, remote_cols,
+            schema=self.impl.schema)
+
+    def create_unique_constraint(self, name, local_cols, **kw):
+        """Issue a "create unique constraint" instruction using the
+        current batch migration context.
+
+        The batch form of this call omits the ``source`` and ``schema``
+        arguments from the call.
+
+        .. seealso::
+
+            :meth:`.Operations.create_unique_constraint`
+
+        """
+        kw['schema'] = self.impl.schema
+        return super(BatchOperations, self).create_unique_constraint(
+            name, self.impl.table_name, local_cols, **kw)
+
+    def create_check_constraint(self, name, condition, **kw):
+        """Issue a "create check constraint" instruction using the
+        current batch migration context.
+
+        The batch form of this call omits the ``source`` and ``schema``
+        arguments from the call.
+
+        .. seealso::
+
+            :meth:`.Operations.create_check_constraint`
+
+        """
+        raise NotImplementedError("not yet implemented")
+
+    def drop_constraint(self, name, type_=None):
+        """Issue a "drop constraint" instruction using the
+        current batch migration context.
+
+        The batch form of this call omits the ``table_name`` and ``schema``
+        arguments from the call.
+
+        .. seealso::
+
+            :meth:`.Operations.drop_constraint`
+
+        """
+        return super(BatchOperations, self).drop_constraint(
+            name, self.impl.table_name, type_=type_,
+            schema=self.impl.schema)
+
+    def create_index(self, name, columns, **kw):
+        """Issue a "create index" instruction using the
+        current batch migration context."""
+
+        kw['schema'] = self.impl.schema
+
+        return super(BatchOperations, self).create_index(
+            name, self.impl.table_name, columns, **kw)
+
+    def drop_index(self, name, **kw):
+        """Issue a "drop index" instruction using the
+        current batch migration context."""
+
+        kw['schema'] = self.impl.schema
+
+        return super(BatchOperations, self).drop_index(
+            name, self.impl.table_name, **kw)
