@@ -1,26 +1,28 @@
 import re
 import sys
-from unittest import TestCase
+from alembic.testing import TestBase
 
-from sqlalchemy import MetaData, Column, Table, Integer, String, Text, \
-    Numeric, CHAR, ForeignKey, DATETIME, INTEGER, \
-    TypeDecorator, CheckConstraint, Unicode, Enum,\
+from sqlalchemy import MetaData, Column, Table, String, \
+    Numeric, CHAR, ForeignKey, DATETIME, Integer, \
+    CheckConstraint, Unicode, Enum, cast,\
     UniqueConstraint, Boolean, ForeignKeyConstraint,\
-    PrimaryKeyConstraint, Index, func
+    PrimaryKeyConstraint, Index, func, text, DefaultClause
+
 from sqlalchemy.types import TIMESTAMP
 from sqlalchemy.dialects import mysql, postgresql
-from sqlalchemy.sql import and_, column, literal_column
+from sqlalchemy.engine.default import DefaultDialect
+from sqlalchemy.sql import and_, column, literal_column, false
 
-from . import patch
+from alembic.testing.mock import patch
 
 from alembic import autogenerate, util, compat
-from . import eq_, eq_ignore_whitespace, requires_092, \
-    requires_09, requires_094
+from alembic.testing import eq_, eq_ignore_whitespace, config
+
 
 py3k = sys.version_info >= (3, )
 
 
-class AutogenRenderTest(TestCase):
+class AutogenRenderTest(TestBase):
 
     """test individual directives"""
 
@@ -93,33 +95,60 @@ class AutogenRenderTest(TestCase):
                 autogenerate.render._add_index(idx, autogen_context),
                 """op.create_index('foo_idx', 't', \
 ['x', 'y'], unique=False, """
-                """postgresql_where=sa.text("t.y = 'something'"))"""
+                """postgresql_where=sa.text(!U"t.y = 'something'"))"""
             )
         else:
             eq_ignore_whitespace(
                 autogenerate.render._add_index(idx, autogen_context),
                 """op.create_index('foo_idx', 't', ['x', 'y'], \
 unique=False, """
-                """postgresql_where=sa.text('t.y = %(y_1)s'))"""
+                """postgresql_where=sa.text(!U't.y = %(y_1)s'))"""
             )
 
-    # def test_render_add_index_func(self):
-    #     """
-    #     autogenerate.render._drop_index using func -- TODO: SQLA needs to
-    #     reflect expressions as well as columns
-    #     """
-    #     m = MetaData()
-    #     t = Table('test', m,
-    #         Column('id', Integer, primary_key=True),
-    #         Column('active', Boolean()),
-    #         Column('code', String(255)),
-    #     )
-    #     idx = Index(
-    #       'test_active_lower_code_idx', t.c.active, func.lower(t.c.code))
-    #     eq_ignore_whitespace(
-    #         autogenerate.render._add_index(idx, self.autogen_context),
-    #         ""
-    #     )
+    @config.requirements.fail_before_sqla_080
+    def test_render_add_index_func(self):
+        m = MetaData()
+        t = Table(
+            'test', m,
+            Column('id', Integer, primary_key=True),
+            Column('code', String(255))
+        )
+        idx = Index('test_lower_code_idx', func.lower(t.c.code))
+        eq_ignore_whitespace(
+            autogenerate.render._add_index(idx, self.autogen_context),
+            "op.create_index('test_lower_code_idx', 'test', "
+            "[sa.text(!U'lower(test.code)')], unique=False)"
+        )
+
+    @config.requirements.fail_before_sqla_080
+    def test_render_add_index_cast(self):
+        m = MetaData()
+        t = Table(
+            'test', m,
+            Column('id', Integer, primary_key=True),
+            Column('code', String(255))
+        )
+        idx = Index('test_lower_code_idx', cast(t.c.code, String))
+        eq_ignore_whitespace(
+            autogenerate.render._add_index(idx, self.autogen_context),
+            "op.create_index('test_lower_code_idx', 'test', "
+            "[sa.text(!U'CAST(test.code AS CHAR)')], unique=False)"
+        )
+
+    @config.requirements.fail_before_sqla_080
+    def test_render_add_index_desc(self):
+        m = MetaData()
+        t = Table(
+            'test', m,
+            Column('id', Integer, primary_key=True),
+            Column('code', String(255))
+        )
+        idx = Index('test_desc_code_idx', t.c.code.desc())
+        eq_ignore_whitespace(
+            autogenerate.render._add_index(idx, self.autogen_context),
+            "op.create_index('test_desc_code_idx', 'test', "
+            "[sa.text(!U'test.code DESC')], unique=False)"
+        )
 
     def test_drop_index(self):
         """
@@ -271,6 +300,32 @@ unique=False, """
             ")"
         )
 
+    def test_render_table_w_unicode_name(self):
+        m = MetaData()
+        t = Table(compat.ue('\u0411\u0435\u0437'), m,
+                  Column('id', Integer, primary_key=True),
+                  )
+        eq_ignore_whitespace(
+            autogenerate.render._add_table(t, self.autogen_context),
+            "op.create_table(%r,"
+            "sa.Column('id', sa.Integer(), nullable=False),"
+            "sa.PrimaryKeyConstraint('id'))" % compat.ue('\u0411\u0435\u0437')
+        )
+
+    def test_render_table_w_unicode_schema(self):
+        m = MetaData()
+        t = Table('test', m,
+                  Column('id', Integer, primary_key=True),
+                  schema=compat.ue('\u0411\u0435\u0437')
+                  )
+        eq_ignore_whitespace(
+            autogenerate.render._add_table(t, self.autogen_context),
+            "op.create_table('test',"
+            "sa.Column('id', sa.Integer(), nullable=False),"
+            "sa.PrimaryKeyConstraint('id'),"
+            "schema=%r)" % compat.ue('\u0411\u0435\u0437')
+        )
+
     @patch("alembic.autogenerate.render.MAX_PYTHON_ARGS", 3)
     def test_render_table_max_cols(self):
         m = MetaData()
@@ -330,7 +385,8 @@ unique=False, """
         eq_ignore_whitespace(
             re.sub(
                 r"u'", "'",
-                autogenerate.render._add_table(t, self.autogen_context)),
+                autogenerate.render._add_table(t, self.autogen_context)
+            ),
             "op.create_table('test',"
             "sa.Column('id', sa.Integer(), nullable=False),"
             "sa.Column('q', sa.Integer(), nullable=True),"
@@ -375,14 +431,14 @@ unique=False, """
         )
 
     def test_render_drop_table(self):
-        eq_(
+        eq_ignore_whitespace(
             autogenerate.render._drop_table(Table("sometable", MetaData()),
                                             self.autogen_context),
             "op.drop_table('sometable')"
         )
 
     def test_render_drop_table_w_schema(self):
-        eq_(
+        eq_ignore_whitespace(
             autogenerate.render._drop_table(
                 Table("sometable", MetaData(), schema='foo'),
                 self.autogen_context),
@@ -406,7 +462,8 @@ unique=False, """
 
         eq_ignore_whitespace(
             autogenerate.render._add_table(t1, self.autogen_context),
-            "op.create_table('t1',sa.Column('x', sa.Integer(), nullable=True))"
+            "op.create_table('t1',"
+            "sa.Column('x', sa.Integer(), nullable=True))"
         )
 
         eq_ignore_whitespace(
@@ -417,7 +474,7 @@ unique=False, """
         )
 
     def test_render_add_column(self):
-        eq_(
+        eq_ignore_whitespace(
             autogenerate.render._add_column(
                 None, "foo", Column("x", Integer, server_default="5"),
                 self.autogen_context),
@@ -426,7 +483,7 @@ unique=False, """
         )
 
     def test_render_add_column_w_schema(self):
-        eq_(
+        eq_ignore_whitespace(
             autogenerate.render._add_column(
                 "foo", "bar", Column("x", Integer, server_default="5"),
                 self.autogen_context),
@@ -435,7 +492,7 @@ unique=False, """
         )
 
     def test_render_drop_column(self):
-        eq_(
+        eq_ignore_whitespace(
             autogenerate.render._drop_column(
                 None, "foo", Column("x", Integer, server_default="5"),
                 self.autogen_context),
@@ -444,7 +501,7 @@ unique=False, """
         )
 
     def test_render_drop_column_w_schema(self):
-        eq_(
+        eq_ignore_whitespace(
             autogenerate.render._drop_column(
                 "foo", "bar", Column("x", Integer, server_default="5"),
                 self.autogen_context),
@@ -460,6 +517,24 @@ unique=False, """
             '"nextval(\'group_to_perm_group_to_perm_id_seq\'::regclass)"'
         )
 
+    def test_render_unicode_server_default(self):
+        default = compat.ue(
+            '\u0411\u0435\u0437 '
+            '\u043d\u0430\u0437\u0432\u0430\u043d\u0438\u044f'
+        )
+
+        c = Column(
+            'x', Unicode,
+            server_default=text(default)
+        )
+
+        eq_ignore_whitespace(
+            autogenerate.render._render_server_default(
+                c.server_default, self.autogen_context
+            ),
+            "sa.text(%r)" % default
+        )
+
     def test_render_col_with_server_default(self):
         c = Column('updated_at', TIMESTAMP(),
                    server_default='TIMEZONE("utc", CURRENT_TIMESTAMP)',
@@ -467,7 +542,7 @@ unique=False, """
         result = autogenerate.render._render_column(
             c, self.autogen_context
         )
-        eq_(
+        eq_ignore_whitespace(
             result,
             'sa.Column(\'updated_at\', sa.TIMESTAMP(), '
             'server_default=\'TIMEZONE("utc", CURRENT_TIMESTAMP)\', '
@@ -480,7 +555,7 @@ unique=False, """
         result = autogenerate.render._render_column(
             c, self.autogen_context
         )
-        eq_(
+        eq_ignore_whitespace(
             result,
             'sa.Column(\'some_key\', sa.Integer(), '
             'autoincrement=False, '
@@ -513,10 +588,11 @@ unique=False, """
         result = autogenerate.render._add_table(
             t, autogen_context
         )
-        eq_(
-            result, """sa.create_table('t',
-col(x),
-render:primary_key\n)"""
+        eq_ignore_whitespace(
+            result,
+            "sa.create_table('t',"
+            "col(x),"
+            "render:primary_key)"
         )
 
     def test_render_modify_type(self):
@@ -572,6 +648,9 @@ render:primary_key\n)"""
         if not util.sqla_08:
             t1.append_constraint(fk)
 
+        # SQLA 0.9 generates a u'' here for remote cols while 0.8 does not,
+        # so just whack out "'u" here from the generated
+
         eq_ignore_whitespace(
             re.sub(
                 r"u'", "'",
@@ -599,7 +678,8 @@ render:primary_key\n)"""
             re.sub(
                 r"u'", "'",
                 autogenerate.render._render_constraint(
-                    fk, self.autogen_context)),
+                    fk, self.autogen_context),
+            ),
             "sa.ForeignKeyConstraint(['c'], ['t2.c_rem'], deferrable=True)"
         )
 
@@ -610,7 +690,8 @@ render:primary_key\n)"""
             re.sub(
                 r"u'", "'",
                 autogenerate.render._render_constraint(
-                    fk, self.autogen_context)),
+                    fk, self.autogen_context)
+            ),
             "sa.ForeignKeyConstraint(['c'], ['t2.c_rem'], initially='XYZ')"
         )
 
@@ -644,7 +725,8 @@ render:primary_key\n)"""
             re.sub(
                 r"u'", "'",
                 autogenerate.render._render_constraint(
-                    fk, self.autogen_context)),
+                    fk, self.autogen_context)
+            ),
             "sa.ForeignKeyConstraint(['c'], ['foo.t2.c_rem'], "
             "onupdate='CASCADE')"
         )
@@ -655,7 +737,7 @@ render:primary_key\n)"""
                 CheckConstraint("im a constraint", name='cc1'),
                 self.autogen_context
             ),
-            "sa.CheckConstraint('im a constraint', name='cc1')"
+            "sa.CheckConstraint(!U'im a constraint', name='cc1')"
         )
 
     def test_render_check_constraint_sqlexpr(self):
@@ -667,7 +749,18 @@ render:primary_key\n)"""
                 CheckConstraint(and_(c > five, c < ten)),
                 self.autogen_context
             ),
-            "sa.CheckConstraint('c > 5 AND c < 10')"
+            "sa.CheckConstraint(!U'c > 5 AND c < 10')"
+        )
+
+    @config.requirements.fail_before_sqla_080
+    def test_render_check_constraint_literal_binds(self):
+        c = column('c')
+        eq_ignore_whitespace(
+            autogenerate.render._render_check_constraint(
+                CheckConstraint(and_(c > 5, c < 10)),
+                self.autogen_context
+            ),
+            "sa.CheckConstraint(!U'c > 5 AND c < 10')"
         )
 
     def test_render_unique_constraint_opts(self):
@@ -681,6 +774,21 @@ render:primary_key\n)"""
             "sa.UniqueConstraint('c', deferrable='XYZ', name='uq_1')"
         )
 
+    def test_add_unique_constraint_unicode_schema(self):
+        m = MetaData()
+        t = Table(
+            't', m, Column('c', Integer),
+            schema=compat.ue('\u0411\u0435\u0437')
+        )
+        eq_ignore_whitespace(
+            autogenerate.render._add_unique_constraint(
+                UniqueConstraint(t.c.c),
+                self.autogen_context
+            ),
+            "op.create_unique_constraint(None, 't', ['c'], "
+            "schema=%r)" % compat.ue('\u0411\u0435\u0437')
+        )
+
     def test_render_modify_nullable_w_default(self):
         eq_ignore_whitespace(
             autogenerate.render._modify_col(
@@ -743,7 +851,7 @@ render:primary_key\n)"""
 
         eq_ignore_whitespace(
             autogenerate.render._repr_type(type_, autogen_context),
-            "sa.MyType()"
+            "tests.test_autogen_render.MyType()"
         )
 
     def test_repr_user_type_user_prefix_present(self):
@@ -769,7 +877,7 @@ render:primary_key\n)"""
             "user.MyType()"
         )
 
-    @requires_09
+    @config.requirements.sqlalchemy_09
     def test_repr_dialect_type(self):
         from sqlalchemy.dialects.mysql import VARCHAR
 
@@ -791,11 +899,114 @@ render:primary_key\n)"""
             set(['from sqlalchemy.dialects import mysql'])
             )
 
+    def test_render_server_default_text(self):
+        c = Column(
+            'updated_at', TIMESTAMP(),
+            server_default=text('now()'),
+            nullable=False)
+        result = autogenerate.render._render_column(
+            c, self.autogen_context
+        )
+        eq_ignore_whitespace(
+            result,
+            'sa.Column(\'updated_at\', sa.TIMESTAMP(), '
+            'server_default=sa.text(!U\'now()\'), '
+            'nullable=False)'
+        )
 
-class RenderNamingConventionTest(TestCase):
+    def test_render_server_default_native_boolean(self):
+        autogen_context = {
+            'opts': {
+                'sqlalchemy_module_prefix': 'sa.',
+                'alembic_module_prefix': 'op.',
+            },
+            'dialect': postgresql.dialect()
+        }
+        c = Column(
+            'updated_at', Boolean(),
+            server_default=false(),
+            nullable=False)
+        result = autogenerate.render._render_column(
+            c, autogen_context,
+        )
+        eq_ignore_whitespace(
+            result,
+            'sa.Column(\'updated_at\', sa.Boolean(), '
+            'server_default=sa.text(!U\'false\'), '
+            'nullable=False)'
+        )
+
+    @config.requirements.fail_before_sqla_09
+    def test_render_server_default_non_native_boolean(self):
+        c = Column(
+            'updated_at', Boolean(),
+            server_default=false(),
+            nullable=False)
+        dialect = DefaultDialect()
+        autogen_context = {
+            'opts': {
+                'sqlalchemy_module_prefix': 'sa.',
+                'alembic_module_prefix': 'op.',
+            },
+            'dialect': dialect
+        }
+
+        result = autogenerate.render._render_column(
+            c, autogen_context
+        )
+        eq_ignore_whitespace(
+            result,
+            'sa.Column(\'updated_at\', sa.Boolean(), '
+            'server_default=sa.text(!U\'0\'), '
+            'nullable=False)'
+        )
+
+    def test_render_server_default_func(self):
+        c = Column(
+            'updated_at', TIMESTAMP(),
+            server_default=func.now(),
+            nullable=False)
+        result = autogenerate.render._render_column(
+            c, self.autogen_context
+        )
+        eq_ignore_whitespace(
+            result,
+            'sa.Column(\'updated_at\', sa.TIMESTAMP(), '
+            'server_default=sa.text(!U\'now()\'), '
+            'nullable=False)'
+        )
+
+    def test_render_server_default_int(self):
+        c = Column(
+            'value', Integer,
+            server_default="0")
+        result = autogenerate.render._render_column(
+            c, self.autogen_context
+        )
+        eq_(
+            result,
+            "sa.Column('value', sa.Integer(), "
+            "server_default='0', nullable=True)"
+        )
+
+    def test_render_modify_reflected_int_server_default(self):
+        eq_ignore_whitespace(
+            autogenerate.render._modify_col(
+                "sometable", "somecolumn",
+                self.autogen_context,
+                existing_type=Integer(),
+                existing_server_default=DefaultClause(text("5")),
+                nullable=True),
+            "op.alter_column('sometable', 'somecolumn', "
+            "existing_type=sa.Integer(), nullable=True, "
+            "existing_server_default=sa.text(!U'5'))"
+        )
+
+
+class RenderNamingConventionTest(TestBase):
+    __requires__ = ('sqlalchemy_094',)
 
     @classmethod
-    @requires_094
     def setup_class(cls):
         cls.autogen_context = {
             'opts': {
@@ -915,7 +1126,7 @@ class RenderNamingConventionTest(TestCase):
         eq_ignore_whitespace(
             autogenerate.render._add_table(t, self.autogen_context),
             "op.create_table('t',sa.Column('c', sa.Integer(), nullable=True),"
-            "sa.CheckConstraint('c > 5', name=op.f('ck_ct_t')))"
+            "sa.CheckConstraint(!U'c > 5', name=op.f('ck_ct_t')))"
         )
 
     def test_inline_fk(self):
@@ -948,5 +1159,6 @@ class RenderNamingConventionTest(TestCase):
                 ck,
                 self.autogen_context
             ),
-            "sa.CheckConstraint('im a constraint', name=op.f('ck_t_cc1'))"
+            "sa.CheckConstraint(!U'im a constraint', name=op.f('ck_t_cc1'))"
         )
+
