@@ -1,20 +1,19 @@
 import re
 import sys
-from unittest import TestCase
-from . import Mock
 
 from sqlalchemy import MetaData, Column, Table, Integer, String, Text, \
-    Numeric, CHAR, ForeignKey, DATETIME, INTEGER, \
-    TypeDecorator, CheckConstraint, Unicode, Enum,\
-    UniqueConstraint, Boolean, ForeignKeyConstraint,\
-    PrimaryKeyConstraint, Index, func
+    Numeric, CHAR, ForeignKey, INTEGER, Index, UniqueConstraint, \
+    TypeDecorator, CheckConstraint, text, PrimaryKeyConstraint
 from sqlalchemy.types import NULLTYPE
 from sqlalchemy.engine.reflection import Inspector
 
 from alembic import autogenerate
 from alembic.migration import MigrationContext
-from . import staging_env, sqlite_db, clear_staging_env, eq_, \
-    db_for_dialect
+from alembic.testing import TestBase
+from alembic.testing import config
+from alembic.testing.mock import Mock
+from alembic.testing.env import staging_env, clear_staging_env
+from alembic.testing import eq_
 
 py3k = sys.version_info >= (3, )
 
@@ -42,7 +41,9 @@ class AutogenTest(object):
 
     @classmethod
     def _get_bind(cls):
-        return sqlite_db()
+        return config.db
+
+    configure_opts = {}
 
     @classmethod
     def setup_class(cls):
@@ -52,52 +53,64 @@ class AutogenTest(object):
         cls.m1.create_all(cls.bind)
         cls.m2 = cls._get_model_schema()
 
-        conn = cls.bind.connect()
-        cls.context = context = MigrationContext.configure(
+    @classmethod
+    def teardown_class(cls):
+        cls.m1.drop_all(cls.bind)
+        clear_staging_env()
+
+    def setUp(self):
+        self.conn = conn = self.bind.connect()
+        ctx_opts = {
+            'compare_type': True,
+            'compare_server_default': True,
+            'target_metadata': self.m2,
+            'upgrade_token': "upgrades",
+            'downgrade_token': "downgrades",
+            'alembic_module_prefix': 'op.',
+            'sqlalchemy_module_prefix': 'sa.',
+        }
+        if self.configure_opts:
+            ctx_opts.update(self.configure_opts)
+        self.context = context = MigrationContext.configure(
             connection=conn,
-            opts={
-                'compare_type': True,
-                'compare_server_default': True,
-                'target_metadata': cls.m2,
-                'upgrade_token': "upgrades",
-                'downgrade_token': "downgrades",
-                'alembic_module_prefix': 'op.',
-                'sqlalchemy_module_prefix': 'sa.',
-            }
+            opts=ctx_opts
         )
 
         connection = context.bind
-        cls.autogen_context = {
+        self.autogen_context = {
             'imports': set(),
             'connection': connection,
             'dialect': connection.dialect,
             'context': context
         }
 
-    @classmethod
-    def teardown_class(cls):
-        cls.m1.drop_all(cls.bind)
-        clear_staging_env()
+    def tearDown(self):
+        self.conn.close()
 
 
 class AutogenFixtureTest(object):
 
-    def _fixture(self, m1, m2, include_schemas=False):
+    def _fixture(
+            self, m1, m2, include_schemas=False,
+            opts=None, object_filters=_default_object_filters):
         self.metadata, model_metadata = m1, m2
         self.metadata.create_all(self.bind)
 
         with self.bind.connect() as conn:
+            ctx_opts = {
+                'compare_type': True,
+                'compare_server_default': True,
+                'target_metadata': model_metadata,
+                'upgrade_token': "upgrades",
+                'downgrade_token': "downgrades",
+                'alembic_module_prefix': 'op.',
+                'sqlalchemy_module_prefix': 'sa.',
+            }
+            if opts:
+                ctx_opts.update(opts)
             self.context = context = MigrationContext.configure(
                 connection=conn,
-                opts={
-                    'compare_type': True,
-                    'compare_server_default': True,
-                    'target_metadata': model_metadata,
-                    'upgrade_token': "upgrades",
-                    'downgrade_token': "downgrades",
-                    'alembic_module_prefix': 'op.',
-                    'sqlalchemy_module_prefix': 'sa.',
-                }
+                opts=ctx_opts
             )
 
             connection = context.bind
@@ -111,7 +124,7 @@ class AutogenFixtureTest(object):
             autogenerate._produce_net_changes(
                 connection, model_metadata, diffs,
                 autogen_context,
-                object_filters=_default_object_filters,
+                object_filters=object_filters,
                 include_schemas=include_schemas
             )
             return diffs
@@ -120,24 +133,16 @@ class AutogenFixtureTest(object):
 
     def setUp(self):
         staging_env()
-        self.bind = self._get_bind()
+        self.bind = config.db
 
     def tearDown(self):
         if hasattr(self, 'metadata'):
             self.metadata.drop_all(self.bind)
         clear_staging_env()
 
-    @classmethod
-    def _get_bind(cls):
-        return sqlite_db()
 
-
-class AutogenCrossSchemaTest(AutogenTest, TestCase):
-
-    @classmethod
-    def _get_bind(cls):
-        cls.test_schema_name = "test_schema"
-        return db_for_dialect('postgresql')
+class AutogenCrossSchemaTest(AutogenTest, TestBase):
+    __only_on__ = 'postgresql'
 
     @classmethod
     def _get_db_schema(cls):
@@ -147,14 +152,14 @@ class AutogenCrossSchemaTest(AutogenTest, TestCase):
               )
         Table('t2', m,
               Column('y', Integer),
-              schema=cls.test_schema_name
+              schema=config.test_schema
               )
         Table('t6', m,
               Column('u', Integer)
               )
         Table('t7', m,
               Column('v', Integer),
-              schema=cls.test_schema_name
+              schema=config.test_schema
               )
 
         return m
@@ -167,14 +172,14 @@ class AutogenCrossSchemaTest(AutogenTest, TestCase):
               )
         Table('t4', m,
               Column('z', Integer),
-              schema=cls.test_schema_name
+              schema=config.test_schema
               )
         Table('t6', m,
               Column('u', Integer)
               )
         Table('t7', m,
               Column('v', Integer),
-              schema=cls.test_schema_name
+              schema=config.test_schema
               )
         return m
 
@@ -212,7 +217,7 @@ class AutogenCrossSchemaTest(AutogenTest, TestCase):
                                           include_schemas=True
                                           )
         eq_(diffs[0][0], "add_table")
-        eq_(diffs[0][1].schema, self.test_schema_name)
+        eq_(diffs[0][1].schema, config.test_schema)
 
     def test_default_schema_omitted_downgrade(self):
         metadata = self.m2
@@ -248,15 +253,11 @@ class AutogenCrossSchemaTest(AutogenTest, TestCase):
                                           include_schemas=True
                                           )
         eq_(diffs[0][0], "remove_table")
-        eq_(diffs[0][1].schema, self.test_schema_name)
+        eq_(diffs[0][1].schema, config.test_schema)
 
 
-class AutogenDefaultSchemaTest(AutogenFixtureTest, TestCase):
-
-    @classmethod
-    def _get_bind(cls):
-        cls.test_schema_name = "test_schema"
-        return db_for_dialect('postgresql')
+class AutogenDefaultSchemaTest(AutogenFixtureTest, TestBase):
+    __only_on__ = 'postgresql'
 
     def test_uses_explcit_schema_in_default_one(self):
 
@@ -308,6 +309,8 @@ class AutogenDefaultSchemaTest(AutogenFixtureTest, TestCase):
 
 
 class ModelOne(object):
+    __requires__ = ('unique_constraint_reflection', )
+
     schema = None
 
     @classmethod
@@ -320,7 +323,8 @@ class ModelOne(object):
               Column('id', Integer, primary_key=True),
               Column('name', String(50)),
               Column('a1', Text),
-              Column("pw", String(50))
+              Column("pw", String(50)),
+              Index('pw_idx', 'pw')
               )
 
         Table('address', m,
@@ -331,7 +335,7 @@ class ModelOne(object):
         Table('order', m,
               Column('order_id', Integer, primary_key=True),
               Column("amount", Numeric(8, 2), nullable=False,
-                     server_default="0"),
+                     server_default=text("0")),
               CheckConstraint('amount >= 0', name='ck_order_amount')
               )
 
@@ -358,12 +362,13 @@ class ModelOne(object):
               Column('id', Integer, primary_key=True),
               Column('email_address', String(100), nullable=False),
               Column('street', String(50)),
+              UniqueConstraint('email_address', name="uq_email")
               )
 
         Table('order', m,
               Column('order_id', Integer, primary_key=True),
               Column('amount', Numeric(10, 2), nullable=True,
-                     server_default="0"),
+                     server_default=text("0")),
               Column('user_id', Integer, ForeignKey('user.id')),
               CheckConstraint('amount > -1', name='ck_order_amount'),
               )
@@ -377,7 +382,8 @@ class ModelOne(object):
         return m
 
 
-class AutogenerateDiffTest(ModelOne, AutogenTest, TestCase):
+class AutogenerateDiffTest(ModelOne, AutogenTest, TestBase):
+    __only_on__ = 'sqlite'
 
     def test_diffs(self):
         """test generation of diff rules"""
@@ -404,20 +410,20 @@ class AutogenerateDiffTest(ModelOne, AutogenTest, TestCase):
         eq_(diffs[2][2], "address")
         eq_(diffs[2][3], metadata.tables['address'].c.street)
 
-        eq_(diffs[3][0], "add_column")
-        eq_(diffs[3][1], None)
-        eq_(diffs[3][2], "order")
-        eq_(diffs[3][3], metadata.tables['order'].c.user_id)
+        eq_(diffs[3][0], "add_constraint")
+        eq_(diffs[3][1].name, "uq_email")
 
-        eq_(diffs[4][0][0], "modify_type")
-        eq_(diffs[4][0][1], None)
-        eq_(diffs[4][0][2], "order")
-        eq_(diffs[4][0][3], "amount")
-        eq_(repr(diffs[4][0][5]), "NUMERIC(precision=8, scale=2)")
-        eq_(repr(diffs[4][0][6]), "Numeric(precision=10, scale=2)")
+        eq_(diffs[4][0], "add_column")
+        eq_(diffs[4][1], None)
+        eq_(diffs[4][2], "order")
+        eq_(diffs[4][3], metadata.tables['order'].c.user_id)
 
-        eq_(diffs[5][0], 'remove_column')
-        eq_(diffs[5][3].name, 'pw')
+        eq_(diffs[5][0][0], "modify_type")
+        eq_(diffs[5][0][1], None)
+        eq_(diffs[5][0][2], "order")
+        eq_(diffs[5][0][3], "amount")
+        eq_(repr(diffs[5][0][5]), "NUMERIC(precision=8, scale=2)")
+        eq_(repr(diffs[5][0][6]), "Numeric(precision=10, scale=2)")
 
         eq_(diffs[6][0][0], "modify_default")
         eq_(diffs[6][0][1], None)
@@ -429,6 +435,12 @@ class AutogenerateDiffTest(ModelOne, AutogenTest, TestCase):
         eq_(diffs[7][0][5], True)
         eq_(diffs[7][0][6], False)
 
+        eq_(diffs[8][0], 'remove_index')
+        eq_(diffs[8][1].name, 'pw_idx')
+
+        eq_(diffs[9][0], 'remove_column')
+        eq_(diffs[9][3].name, 'pw')
+
     def test_render_nothing(self):
         context = MigrationContext.configure(
             connection=self.bind.connect(),
@@ -452,6 +464,34 @@ class AutogenerateDiffTest(ModelOne, AutogenTest, TestCase):
     pass
     ### end Alembic commands ###""")
 
+    def test_render_nothing_batch(self):
+        context = MigrationContext.configure(
+            connection=self.bind.connect(),
+            opts={
+                'compare_type': True,
+                'compare_server_default': True,
+                'target_metadata': self.m1,
+                'upgrade_token': "upgrades",
+                'downgrade_token': "downgrades",
+                'alembic_module_prefix': 'op.',
+                'sqlalchemy_module_prefix': 'sa.',
+                'render_as_batch': True
+            }
+        )
+        template_args = {}
+        autogenerate._produce_migration_diffs(
+            context, template_args, set(),
+            include_symbol=lambda name, schema: False
+        )
+        eq_(re.sub(r"u'", "'", template_args['upgrades']),
+            """### commands auto generated by Alembic - please adjust! ###
+    pass
+    ### end Alembic commands ###""")
+        eq_(re.sub(r"u'", "'", template_args['downgrades']),
+            """### commands auto generated by Alembic - please adjust! ###
+    pass
+    ### end Alembic commands ###""")
+
     def test_render_diffs_standard(self):
         """test a full render including indentation"""
 
@@ -472,13 +512,13 @@ class AutogenerateDiffTest(ModelOne, AutogenTest, TestCase):
     op.drop_table('extra')
     op.add_column('address', sa.Column('street', sa.String(length=50), \
 nullable=True))
+    op.create_unique_constraint('uq_email', 'address', ['email_address'])
     op.add_column('order', sa.Column('user_id', sa.Integer(), nullable=True))
     op.alter_column('order', 'amount',
                existing_type=sa.NUMERIC(precision=8, scale=2),
                type_=sa.Numeric(precision=10, scale=2),
                nullable=True,
-               existing_server_default='0')
-    op.drop_column('user', 'pw')
+               existing_server_default=sa.text('0'))
     op.alter_column('user', 'a1',
                existing_type=sa.TEXT(),
                server_default='x',
@@ -486,10 +526,15 @@ nullable=True))
     op.alter_column('user', 'name',
                existing_type=sa.VARCHAR(length=50),
                nullable=False)
+    op.drop_index('pw_idx', table_name='user')
+    op.drop_column('user', 'pw')
     ### end Alembic commands ###""")
 
         eq_(re.sub(r"u'", "'", template_args['downgrades']),
             """### commands auto generated by Alembic - please adjust! ###
+    op.add_column('user', sa.Column('pw', sa.VARCHAR(length=50), \
+nullable=True))
+    op.create_index('pw_idx', 'user', ['pw'], unique=False)
     op.alter_column('user', 'name',
                existing_type=sa.VARCHAR(length=50),
                nullable=True)
@@ -497,14 +542,13 @@ nullable=True))
                existing_type=sa.TEXT(),
                server_default=None,
                existing_nullable=True)
-    op.add_column('user', sa.Column('pw', sa.VARCHAR(length=50), \
-nullable=True))
     op.alter_column('order', 'amount',
                existing_type=sa.Numeric(precision=10, scale=2),
                type_=sa.NUMERIC(precision=8, scale=2),
                nullable=False,
-               existing_server_default='0')
+               existing_server_default=sa.text('0'))
     op.drop_column('order', 'user_id')
+    op.drop_constraint('uq_email', 'address')
     op.drop_column('address', 'street')
     op.create_table('extra',
     sa.Column('x', sa.CHAR(), nullable=True),
@@ -514,6 +558,83 @@ nullable=True))
     op.drop_table('item')
     ### end Alembic commands ###""")
 
+    def test_render_diffs_batch(self):
+        """test a full render in batch mode including indentation"""
+
+        template_args = {}
+        self.context.opts['render_as_batch'] = True
+        autogenerate._produce_migration_diffs(
+            self.context, template_args, set())
+
+        eq_(re.sub(r"u'", "'", template_args['upgrades']),
+            """### commands auto generated by Alembic - please adjust! ###
+    op.create_table('item',
+    sa.Column('id', sa.Integer(), nullable=False),
+    sa.Column('description', sa.String(length=100), nullable=True),
+    sa.Column('order_id', sa.Integer(), nullable=True),
+    sa.CheckConstraint('len(description) > 5'),
+    sa.ForeignKeyConstraint(['order_id'], ['order.order_id'], ),
+    sa.PrimaryKeyConstraint('id')
+    )
+    op.drop_table('extra')
+    with op.batch_alter_table('address', schema=None) as batch_op:
+        batch_op.add_column(sa.Column('street', sa.String(length=50), nullable=True))
+        batch_op.create_unique_constraint('uq_email', ['email_address'])
+
+    with op.batch_alter_table('order', schema=None) as batch_op:
+        batch_op.add_column(sa.Column('user_id', sa.Integer(), nullable=True))
+        batch_op.alter_column('amount',
+               existing_type=sa.NUMERIC(precision=8, scale=2),
+               type_=sa.Numeric(precision=10, scale=2),
+               nullable=True,
+               existing_server_default=sa.text('0'))
+
+    with op.batch_alter_table('user', schema=None) as batch_op:
+        batch_op.alter_column('a1',
+               existing_type=sa.TEXT(),
+               server_default='x',
+               existing_nullable=True)
+        batch_op.alter_column('name',
+               existing_type=sa.VARCHAR(length=50),
+               nullable=False)
+        batch_op.drop_index('pw_idx')
+        batch_op.drop_column('pw')
+
+    ### end Alembic commands ###""")
+
+        eq_(re.sub(r"u'", "'", template_args['downgrades']),
+            """### commands auto generated by Alembic - please adjust! ###
+    with op.batch_alter_table('user', schema=None) as batch_op:
+        batch_op.add_column(sa.Column('pw', sa.VARCHAR(length=50), nullable=True))
+        batch_op.create_index('pw_idx', ['pw'], unique=False)
+        batch_op.alter_column('name',
+               existing_type=sa.VARCHAR(length=50),
+               nullable=True)
+        batch_op.alter_column('a1',
+               existing_type=sa.TEXT(),
+               server_default=None,
+               existing_nullable=True)
+
+    with op.batch_alter_table('order', schema=None) as batch_op:
+        batch_op.alter_column('amount',
+               existing_type=sa.Numeric(precision=10, scale=2),
+               type_=sa.NUMERIC(precision=8, scale=2),
+               nullable=False,
+               existing_server_default=sa.text('0'))
+        batch_op.drop_column('user_id')
+
+    with op.batch_alter_table('address', schema=None) as batch_op:
+        batch_op.drop_constraint('uq_email')
+        batch_op.drop_column('street')
+
+    op.create_table('extra',
+    sa.Column('x', sa.CHAR(), nullable=True),
+    sa.Column('uid', sa.INTEGER(), nullable=True),
+    sa.ForeignKeyConstraint(['uid'], ['user.id'], )
+    )
+    op.drop_table('item')
+    ### end Alembic commands ###""")
+
     def test_include_symbol(self):
         context = MigrationContext.configure(
             connection=self.bind.connect(),
@@ -640,13 +761,10 @@ nullable=True))
         )
 
 
-class AutogenerateDiffTestWSchema(ModelOne, AutogenTest, TestCase):
+class AutogenerateDiffTestWSchema(ModelOne, AutogenTest, TestBase):
+    __only_on__ = 'postgresql'
     schema = "test_schema"
 
-    @classmethod
-    def _get_bind(cls):
-        return db_for_dialect('postgresql')
-
     def test_diffs(self):
         """test generation of diff rules"""
 
@@ -673,20 +791,20 @@ class AutogenerateDiffTestWSchema(ModelOne, AutogenTest, TestCase):
         eq_(diffs[2][2], "address")
         eq_(diffs[2][3], metadata.tables['%s.address' % self.schema].c.street)
 
-        eq_(diffs[3][0], "add_column")
-        eq_(diffs[3][1], self.schema)
-        eq_(diffs[3][2], "order")
-        eq_(diffs[3][3], metadata.tables['%s.order' % self.schema].c.user_id)
+        eq_(diffs[3][0], "add_constraint")
+        eq_(diffs[3][1].name, "uq_email")
 
-        eq_(diffs[4][0][0], "modify_type")
-        eq_(diffs[4][0][1], self.schema)
-        eq_(diffs[4][0][2], "order")
-        eq_(diffs[4][0][3], "amount")
-        eq_(repr(diffs[4][0][5]), "NUMERIC(precision=8, scale=2)")
-        eq_(repr(diffs[4][0][6]), "Numeric(precision=10, scale=2)")
+        eq_(diffs[4][0], "add_column")
+        eq_(diffs[4][1], self.schema)
+        eq_(diffs[4][2], "order")
+        eq_(diffs[4][3], metadata.tables['%s.order' % self.schema].c.user_id)
 
-        eq_(diffs[5][0], 'remove_column')
-        eq_(diffs[5][3].name, 'pw')
+        eq_(diffs[5][0][0], "modify_type")
+        eq_(diffs[5][0][1], self.schema)
+        eq_(diffs[5][0][2], "order")
+        eq_(diffs[5][0][3], "amount")
+        eq_(repr(diffs[5][0][5]), "NUMERIC(precision=8, scale=2)")
+        eq_(repr(diffs[5][0][6]), "Numeric(precision=10, scale=2)")
 
         eq_(diffs[6][0][0], "modify_default")
         eq_(diffs[6][0][1], self.schema)
@@ -698,6 +816,12 @@ class AutogenerateDiffTestWSchema(ModelOne, AutogenTest, TestCase):
         eq_(diffs[7][0][5], True)
         eq_(diffs[7][0][6], False)
 
+        eq_(diffs[8][0], 'remove_index')
+        eq_(diffs[8][1].name, 'pw_idx')
+
+        eq_(diffs[9][0], 'remove_column')
+        eq_(diffs[9][3].name, 'pw')
+
     def test_render_nothing(self):
         context = MigrationContext.configure(
             connection=self.bind.connect(),
@@ -749,15 +873,16 @@ class AutogenerateDiffTestWSchema(ModelOne, AutogenTest, TestCase):
     op.drop_table('extra', schema='%(schema)s')
     op.add_column('address', sa.Column('street', sa.String(length=50), \
 nullable=True), schema='%(schema)s')
+    op.create_unique_constraint('uq_email', 'address', ['email_address'], \
+schema='test_schema')
     op.add_column('order', sa.Column('user_id', sa.Integer(), nullable=True), \
 schema='%(schema)s')
     op.alter_column('order', 'amount',
                existing_type=sa.NUMERIC(precision=8, scale=2),
                type_=sa.Numeric(precision=10, scale=2),
                nullable=True,
-               existing_server_default='0::numeric',
+               existing_server_default=sa.text('0'),
                schema='%(schema)s')
-    op.drop_column('user', 'pw', schema='%(schema)s')
     op.alter_column('user', 'a1',
                existing_type=sa.TEXT(),
                server_default='x',
@@ -767,10 +892,15 @@ schema='%(schema)s')
                existing_type=sa.VARCHAR(length=50),
                nullable=False,
                schema='%(schema)s')
+    op.drop_index('pw_idx', table_name='user', schema='test_schema')
+    op.drop_column('user', 'pw', schema='%(schema)s')
     ### end Alembic commands ###""" % {"schema": self.schema})
 
         eq_(re.sub(r"u'", "'", template_args['downgrades']),
             """### commands auto generated by Alembic - please adjust! ###
+    op.add_column('user', sa.Column('pw', sa.VARCHAR(length=50), \
+autoincrement=False, nullable=True), schema='%(schema)s')
+    op.create_index('pw_idx', 'user', ['pw'], unique=False, schema='%(schema)s')
     op.alter_column('user', 'name',
                existing_type=sa.VARCHAR(length=50),
                nullable=True,
@@ -780,15 +910,14 @@ schema='%(schema)s')
                server_default=None,
                existing_nullable=True,
                schema='%(schema)s')
-    op.add_column('user', sa.Column('pw', sa.VARCHAR(length=50), \
-autoincrement=False, nullable=True), schema='%(schema)s')
     op.alter_column('order', 'amount',
                existing_type=sa.Numeric(precision=10, scale=2),
                type_=sa.NUMERIC(precision=8, scale=2),
                nullable=False,
-               existing_server_default='0::numeric',
+               existing_server_default=sa.text('0'),
                schema='%(schema)s')
     op.drop_column('order', 'user_id', schema='%(schema)s')
+    op.drop_constraint('uq_email', 'address', schema='test_schema')
     op.drop_column('address', 'street', schema='%(schema)s')
     op.create_table('extra',
     sa.Column('x', sa.CHAR(length=1), autoincrement=False, nullable=True),
@@ -801,7 +930,8 @@ name='extra_uid_fkey'),
     ### end Alembic commands ###""" % {"schema": self.schema})
 
 
-class AutogenerateCustomCompareTypeTest(AutogenTest, TestCase):
+class AutogenerateCustomCompareTypeTest(AutogenTest, TestBase):
+    __only_on__ = 'sqlite'
 
     @classmethod
     def _get_db_schema(cls):
@@ -868,7 +998,40 @@ class AutogenerateCustomCompareTypeTest(AutogenTest, TestCase):
         eq_(diffs[1][0][0], 'modify_type')
 
 
-class AutogenKeyTest(AutogenTest, TestCase):
+class PKConstraintUpgradesIgnoresNullableTest(AutogenTest, TestBase):
+    __backend__ = True
+
+    # test workaround for SQLAlchemy issue #3023, alembic issue #199
+    @classmethod
+    def _get_db_schema(cls):
+        m = MetaData()
+
+        Table(
+            'person_to_role', m,
+            Column('person_id', Integer, autoincrement=False),
+            Column('role_id', Integer, autoincrement=False),
+            PrimaryKeyConstraint('person_id', 'role_id')
+        )
+        return m
+
+    @classmethod
+    def _get_model_schema(cls):
+        return cls._get_db_schema()
+
+    def test_no_change(self):
+        metadata = self.m2
+        connection = self.context.bind
+
+        diffs = []
+
+        autogenerate._produce_net_changes(connection, metadata, diffs,
+                                          self.autogen_context
+                                          )
+        eq_(diffs, [])
+
+
+class AutogenKeyTest(AutogenTest, TestBase):
+    __only_on__ = 'sqlite'
 
     @classmethod
     def _get_db_schema(cls):
@@ -913,7 +1076,63 @@ class AutogenKeyTest(AutogenTest, TestCase):
         eq_(diffs[1][3].key, "otherkey")
 
 
-class AutogenerateDiffOrderTest(AutogenTest, TestCase):
+class AutogenVersionTableTest(AutogenTest, TestBase):
+    __only_on__ = 'sqlite'
+    version_table_name = 'alembic_version'
+    version_table_schema = None
+
+    @classmethod
+    def _get_db_schema(cls):
+        m = MetaData()
+        Table(
+            cls.version_table_name, m,
+            Column('x', Integer), schema=cls.version_table_schema)
+        return m
+
+    @classmethod
+    def _get_model_schema(cls):
+        m = MetaData()
+        return m
+
+    def test_no_version_table(self):
+        diffs = []
+        autogenerate._produce_net_changes(self.context.bind, self.m2,
+                                          diffs, self.autogen_context)
+        eq_(diffs, [])
+
+    def test_version_table_in_target(self):
+        diffs = []
+        Table(
+            self.version_table_name,
+            self.m2, Column('x', Integer), schema=self.version_table_schema)
+
+        autogenerate._produce_net_changes(self.context.bind, self.m2,
+                                          diffs, self.autogen_context)
+        eq_(diffs, [])
+
+
+class AutogenCustomVersionTableSchemaTest(AutogenVersionTableTest):
+    __only_on__ = 'postgresql'
+    version_table_schema = 'test_schema'
+    configure_opts = {'version_table_schema': 'test_schema'}
+
+
+class AutogenCustomVersionTableTest(AutogenVersionTableTest):
+    version_table_name = 'my_version_table'
+    configure_opts = {'version_table': 'my_version_table'}
+
+
+class AutogenCustomVersionTableAndSchemaTest(AutogenVersionTableTest):
+    __only_on__ = 'postgresql'
+    version_table_name = 'my_version_table'
+    version_table_schema = 'test_schema'
+    configure_opts = {
+        'version_table': 'my_version_table',
+        'version_table_schema': 'test_schema'}
+
+
+class AutogenerateDiffOrderTest(AutogenTest, TestBase):
+    __only_on__ = 'sqlite'
 
     @classmethod
     def _get_db_schema(cls):
@@ -952,7 +1171,8 @@ class AutogenerateDiffOrderTest(AutogenTest, TestCase):
         eq_(diffs[1][1].name, "child")
 
 
-class CompareMetadataTest(ModelOne, AutogenTest, TestCase):
+class CompareMetadataTest(ModelOne, AutogenTest, TestBase):
+    __only_on__ = 'sqlite'
 
     def test_compare_metadata(self):
         metadata = self.m2
@@ -972,20 +1192,20 @@ class CompareMetadataTest(ModelOne, AutogenTest, TestCase):
         eq_(diffs[2][2], "address")
         eq_(diffs[2][3], metadata.tables['address'].c.street)
 
-        eq_(diffs[3][0], "add_column")
-        eq_(diffs[3][1], None)
-        eq_(diffs[3][2], "order")
-        eq_(diffs[3][3], metadata.tables['order'].c.user_id)
+        eq_(diffs[3][0], "add_constraint")
+        eq_(diffs[3][1].name, "uq_email")
 
-        eq_(diffs[4][0][0], "modify_type")
-        eq_(diffs[4][0][1], None)
-        eq_(diffs[4][0][2], "order")
-        eq_(diffs[4][0][3], "amount")
-        eq_(repr(diffs[4][0][5]), "NUMERIC(precision=8, scale=2)")
-        eq_(repr(diffs[4][0][6]), "Numeric(precision=10, scale=2)")
+        eq_(diffs[4][0], "add_column")
+        eq_(diffs[4][1], None)
+        eq_(diffs[4][2], "order")
+        eq_(diffs[4][3], metadata.tables['order'].c.user_id)
 
-        eq_(diffs[5][0], 'remove_column')
-        eq_(diffs[5][3].name, 'pw')
+        eq_(diffs[5][0][0], "modify_type")
+        eq_(diffs[5][0][1], None)
+        eq_(diffs[5][0][2], "order")
+        eq_(diffs[5][0][3], "amount")
+        eq_(repr(diffs[5][0][5]), "NUMERIC(precision=8, scale=2)")
+        eq_(repr(diffs[5][0][6]), "Numeric(precision=10, scale=2)")
 
         eq_(diffs[6][0][0], "modify_default")
         eq_(diffs[6][0][1], None)
@@ -997,6 +1217,12 @@ class CompareMetadataTest(ModelOne, AutogenTest, TestCase):
         eq_(diffs[7][0][5], True)
         eq_(diffs[7][0][6], False)
 
+        eq_(diffs[8][0], 'remove_index')
+        eq_(diffs[8][1].name, 'pw_idx')
+
+        eq_(diffs[9][0], 'remove_column')
+        eq_(diffs[9][3].name, 'pw')
+
     def test_compare_metadata_include_object(self):
         metadata = self.m2
 
@@ -1065,13 +1291,10 @@ class CompareMetadataTest(ModelOne, AutogenTest, TestCase):
         eq_(diffs[2][1][6], True)
 
 
-class PGCompareMetaData(ModelOne, AutogenTest, TestCase):
+class PGCompareMetaData(ModelOne, AutogenTest, TestBase):
+    __only_on__ = 'postgresql'
     schema = "test_schema"
 
-    @classmethod
-    def _get_bind(cls):
-        return db_for_dialect('postgresql')
-
     def test_compare_metadata_schema(self):
         metadata = self.m2
 
@@ -1097,11 +1320,14 @@ class PGCompareMetaData(ModelOne, AutogenTest, TestCase):
         eq_(diffs[2][2], "address")
         eq_(diffs[2][3], metadata.tables['test_schema.address'].c.street)
 
-        eq_(diffs[3][0], "add_column")
-        eq_(diffs[3][1], "test_schema")
-        eq_(diffs[3][2], "order")
-        eq_(diffs[3][3], metadata.tables['test_schema.order'].c.user_id)
+        eq_(diffs[3][0], "add_constraint")
+        eq_(diffs[3][1].name, "uq_email")
 
-        eq_(diffs[4][0][0], 'modify_nullable')
-        eq_(diffs[4][0][5], False)
-        eq_(diffs[4][0][6], True)
+        eq_(diffs[4][0], "add_column")
+        eq_(diffs[4][1], "test_schema")
+        eq_(diffs[4][2], "order")
+        eq_(diffs[4][3], metadata.tables['test_schema.order'].c.user_id)
+
+        eq_(diffs[5][0][0], 'modify_nullable')
+        eq_(diffs[5][0][5], False)
+        eq_(diffs[5][0][6], True)
